/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	"lxrun.h"
#include	"linux/errno.h"

/*
 * convert OS error numbers into Linux error numbers
 */

/*
 * the following errors (which probably never happen)
 * have no Linux equivalent and are mapped to LX_EINVAL
 *
 * ELBIN
 * EINIT
 * EREMDEV
 */

int
lx_errno(int err)
{
	switch (err)
	{
		case EPERM:		return LX_EPERM;
		case ENOENT:		return LX_ENOENT;
		case ESRCH:		return LX_ESRCH;
		case EINTR:		return LX_EINTR;
		case EIO:		return LX_EIO;
		case ENXIO:		return LX_ENXIO;
		case E2BIG:		return LX_E2BIG;
		case ENOEXEC:		return LX_ENOEXEC;
		case EBADF:		return LX_EBADF;
		case ECHILD:		return LX_ECHILD;
		case EAGAIN:		return LX_EAGAIN;
		case ENOMEM:		return LX_ENOMEM;
		case EACCES:		return LX_EACCES;
		case EFAULT:		return LX_EFAULT;
		case ENOTBLK:		return LX_ENOTBLK;
		case EBUSY:		return LX_EBUSY;
		case EEXIST:		return LX_EEXIST;
		case EXDEV:		return LX_EXDEV;
		case ENODEV:		return LX_ENODEV;
		case ENOTDIR:		return LX_ENOTDIR;
		case EISDIR:		return LX_EISDIR;
		case EINVAL:		return LX_EINVAL;
		case ENFILE:		return LX_ENFILE;
		case EMFILE:		return LX_EMFILE;
		case ENOTTY:		return LX_ENOTTY;
		case ETXTBSY:		return LX_ETXTBSY;
		case EFBIG:		return LX_EFBIG;
		case ENOSPC:		return LX_ENOSPC;
		case ESPIPE:		return LX_ESPIPE;
		case EROFS:		return LX_EROFS;
		case EMLINK:		return LX_EMLINK;
		case EPIPE:		return LX_EPIPE;
		case EDOM:		return LX_EDOM;
		case ERANGE:		return LX_ERANGE;
		case ENOMSG:		return LX_ENOMSG;
		case EIDRM:		return LX_EIDRM;
		case ECHRNG:		return LX_ECHRNG;
		case EL2NSYNC:		return LX_EL2NSYNC;
		case EL3HLT:		return LX_EL3HLT;
		case EL3RST:		return LX_EL3RST;
		case ELNRNG:		return LX_ELNRNG;
		case EUNATCH:		return LX_EUNATCH;
		case ENOCSI:		return LX_ENOCSI;
		case EL2HLT:		return LX_EL2HLT;
		case EDEADLK:		return LX_EDEADLK;
		case ENOLCK:		return LX_ENOLCK;
		case EBADE:		return LX_EBADE;
		case EBADR:		return LX_EBADR;
		case EXFULL:		return LX_EXFULL;
		case ENOANO:		return LX_ENOANO;
		case EBADRQC:		return LX_EBADRQC;
		case EBADSLT:		return LX_EBADSLT;
		case EDEADLOCK:		return LX_EDEADLOCK;
		case EBFONT:		return LX_EBFONT;
		case ENOSTR:		return LX_ENODATA;
		case ENODATA:		return LX_ENODATA;
		case ETIME:		return LX_ETIME;
		case ENOSR:		return LX_ENOSR;
		case ENONET:		return LX_ENONET;
		case ENOPKG:		return LX_ENOPKG;
		case EREMOTE:		return LX_EREMOTE;
		case ENOLINK:		return LX_ENOLINK;
		case EADV:		return LX_EADV;
		case ESRMNT:		return LX_ESRMNT;
		case ECOMM:		return LX_ECOMM;
		case EPROTO:		return LX_EPROTO;
		case EMULTIHOP:		return LX_EMULTIHOP;
#if defined(OSR5)
		case ELBIN:		return LX_EINVAL;
		case EDOTDOT:		return LX_EDOTDOT;
#endif
		case EBADMSG:		return LX_EBADMSG;
		case ENAMETOOLONG:	return LX_ENAMETOOLONG;
		case EOVERFLOW:		return LX_EOVERFLOW;
		case ENOTUNIQ:		return LX_ENOTUNIQ;
		case EBADFD:		return LX_EBADFD;
		case EREMCHG:		return LX_EREMCHG;
		case ELIBACC:		return LX_ELIBACC;
		case ELIBBAD:		return LX_ELIBBAD;
		case ELIBSCN:		return LX_ELIBSCN;
		case ELIBMAX:		return LX_ELIBMAX;
		case ELIBEXEC:		return LX_ELIBEXEC;
		case EILSEQ:		return LX_EILSEQ;
		case ENOSYS:		return LX_ENOSYS;
#if defined(OSR5)
		case EWOULDBLOCK:	return LX_EWOULDBLOCK;
#endif
		case EINPROGRESS:	return LX_EINPROGRESS;
		case EALREADY:		return LX_EALREADY;
		case ENOTSOCK:		return LX_ENOTSOCK;
		case EDESTADDRREQ:	return LX_EDESTADDRREQ;
		case EMSGSIZE:		return LX_EMSGSIZE;
		case EPROTOTYPE:	return LX_EPROTOTYPE;
		case EPROTONOSUPPORT:	return LX_EPROTONOSUPPORT;
		case ESOCKTNOSUPPORT:	return LX_ESOCKTNOSUPPORT;
		case EOPNOTSUPP:	return LX_EOPNOTSUPP;
		case EPFNOSUPPORT:	return LX_EPFNOSUPPORT;
		case EAFNOSUPPORT:	return LX_EAFNOSUPPORT;
		case EADDRINUSE:	return LX_EADDRINUSE;
		case EADDRNOTAVAIL:	return LX_EADDRNOTAVAIL;
		case ENETDOWN:		return LX_ENETDOWN;
		case ENETUNREACH:	return LX_ENETUNREACH;
		case ENETRESET:		return LX_ENETRESET;
		case ECONNABORTED:	return LX_ECONNABORTED;
		case ECONNRESET:	return LX_ECONNRESET;
		case EISCONN:		return LX_EISCONN;
		case ENOTCONN:		return LX_ENOTCONN;
		case ESHUTDOWN:		return LX_ESHUTDOWN;
		case ETOOMANYREFS:	return LX_ETOOMANYREFS;
		case ETIMEDOUT:		return LX_ETIMEDOUT;
		case ECONNREFUSED:	return LX_ECONNREFUSED;
		case EHOSTDOWN:		return LX_EHOSTDOWN;
		case EHOSTUNREACH:	return LX_EHOSTUNREACH;
		case ENOPROTOOPT:	return LX_ENOPROTOOPT;
		case EUCLEAN:		return LX_EUCLEAN;
		case ENOTNAM:		return LX_ENOTNAM;
		case ENAVAIL:		return LX_ENAVAIL;
		case EISNAM:		return LX_EISNAM;
		case EREMOTEIO:		return LX_EREMOTEIO;
		case EINIT:		return LX_EINVAL;
		case EREMDEV:		return LX_EINVAL;
		case ENOTEMPTY:		return LX_ENOTEMPTY;
		case ELOOP:		return LX_ELOOP;
		case ESTALE:		return LX_ESTALE;
		case ERESTART:		return LX_ERESTART;
		case ESTRPIPE:		return LX_ESTRPIPE;
	}
	return LX_EINVAL;
}
