#if !defined(LX_SOCKET_H)
#define LX_SOCKET_H

/*
 * Socket types.
 */
#define LX_SOCK_STREAM	1		/* stream (connection) socket	*/
#define LX_SOCK_DGRAM	2		/* datagram (conn.less) socket	*/
#define LX_SOCK_RAW	3		/* raw socket			*/
#define LX_SOCK_RDM	4		/* reliably-delivered message	*/
#define LX_SOCK_SEQPACKET	5	/* sequential packet socket	*/
#define LX_SOCK_PACKET	10		/* linux specific way of	*/
					/* getting packets at the dev	*/
					/* level.  For writing rarp and	*/
					/* other similar things on the	*/
					/* user level.			*/

/*
 * Supported address families.
 */
#define LX_AF_UNSPEC	0
#define LX_AF_UNIX	1
#define LX_AF_INET	2
#define LX_AF_AX25	3
#define LX_AF_IPX	4

#define LX_AF_MAX	8

/*
 * protocol families, same as address families
 */
#define LX_PF_UNSPEC	LX_AF_UNSPEC
#define LX_PF_UNIX	LX_AF_UNIX
#define LX_PF_INET	LX_AF_INET
#define LX_PF_AX25	LX_AF_AX25
#define LX_PF_IPX	LX_AF_IPX

#define LX_PF_MAX	LX_AF_MAX

/* Socket-level I/O control calls. */
#define LX_FIOSETOWN 	0x8901
#define LX_SIOCSPGRP	0x8902
#define LX_FIOGETOWN	0x8903
#define LX_SIOCGPGRP	0x8904
#define LX_SIOCATMARK	0x8905
#define LX_SIOCGSTAMP	0x8906		/* Get stamp */
#define LX_SIOCGIFCONF		0x8912		/* Get interface conf */
#define LX_SIOCGIFFLAGS		0x8913		/* Get interface flags */
#define LX_SIOCGIFADDR		0x8915		/* Get interface address */
#define LX_SIOCGIFBRDADDR	0x8919		/* Get broadcast address */
#define LX_SIOCGIFNETMASK	0x891b		/* Get network PA mask */

/* For setsockoptions(2) */
#define LX_SOL_SOCKET	1

#define LX_SO_DEBUG	1
#define LX_SO_REUSEADDR	2
#define LX_SO_TYPE		3
#define LX_SO_ERROR	4
#define LX_SO_DONTROUTE	5
#define LX_SO_BROADCAST	6
#define LX_SO_SNDBUF	7
#define LX_SO_RCVBUF	8
#define LX_SO_KEEPALIVE	9
#define LX_SO_OOBINLINE	10
#define LX_SO_NO_CHECK	11
#define LX_SO_PRIORITY	12
#define LX_SO_LINGER	13
/* To add :#define LX_SO_REUSEPORT 14 */

/*
 * socketcall() subfunctions
 */
#define LX_SYS_SOCKET		1
#define LX_SYS_BIND		2
#define LX_SYS_CONNECT		3
#define LX_SYS_LISTEN		4
#define LX_SYS_ACCEPT		5
#define LX_SYS_GETSOCKNAME	6
#define LX_SYS_GETPEERNAME	7
#define LX_SYS_SOCKETPAIR	8
#define LX_SYS_SEND		9
#define LX_SYS_RECV		10
#define LX_SYS_SENDTO		11
#define LX_SYS_RECVFROM		12
#define LX_SYS_SHUTDOWN		13
#define LX_SYS_SETSOCKOPT	14
#define LX_SYS_GETSOCKOPT	15


#define LX_SO_ACCEPTCON	(1<<16)		/* performed a listen		*/
#define LX_SO_WAITDATA	(1<<17)		/* wait data to read		*/
#define LX_SO_NOSPACE	(1<<18)		/* no space to write		*/

#endif
