#if !defined(LX_TERMIOS_H)
#define LX_TERMIOS_H

#define LX_TCGETS	0x5401
#define LX_TCSETS	0x5402
#define LX_TCSETSW	0x5403
#define LX_TCSETSF	0x5404
#define LX_TCGETA	0x5405
#define LX_TCSETA	0x5406
#define LX_TCSETAW	0x5407
#define LX_TCSETAF	0x5408
#define LX_TCSBRK	0x5409
#define LX_TCXONC	0x540A
#define LX_TCFLSH	0x540B
#define LX_TIOCEXCL	0x540C
#define LX_TIOCNXCL	0x540D
#define LX_TIOCSCTTY	0x540E
#define LX_TIOCGPGRP	0x540F
#define LX_TIOCSPGRP	0x5410
#define LX_TIOCOUTQ	0x5411
#define LX_TIOCSTI	0x5412
#define LX_TIOCGWINSZ	0x5413
#define LX_TIOCSWINSZ	0x5414
#define LX_TIOCMGET	0x5415
#define LX_TIOCMBIS	0x5416
#define LX_TIOCMBIC	0x5417
#define LX_TIOCMSET	0x5418
#define LX_TIOCGSOFTCAR	0x5419
#define LX_TIOCSSOFTCAR	0x541A
#define LX_FIONREAD	0x541B
#define LX_TIOCINQ	LX_FIONREAD
#define LX_TIOCLINUX	0x541C
#define LX_TIOCCONS	0x541D
#define LX_TIOCGSERIAL	0x541E
#define LX_TIOCSSERIAL	0x541F
#define LX_TIOCPKT	0x5420
#define LX_FIONBIO	0x5421
#define LX_TIOCNOTTY	0x5422
#define LX_TIOCSETD	0x5423
#define LX_TIOCGETD	0x5424
#define LX_TCSBRKP	0x5425	/* Needed for POSIX tcsendbreak() */
#define LX_TIOCTTYGSTRUCT	0x5426  /* For debugging only */
#define LX_FIONCLEX	0x5450  /* these numbers need to be adjusted. */
#define LX_FIOCLEX	0x5451
#define LX_FIOASYNC	0x5452
#define LX_TIOCSERCONFIG	0x5453
#define LX_TIOCSERGWILD	0x5454
#define LX_TIOCSERSWILD	0x5455
#define LX_TIOCGLCKTRMIOS	0x5456
#define LX_TIOCSLCKTRMIOS	0x5457
#define LX_TIOCSERGSTRUCT	0x5458 /* For debugging only */
#define LX_TIOCSERGETLSR   0x5459 /* Get line status register */
#define LX_TIOCSERGETMULTI 0x545A /* Get multiport config  */
#define LX_TIOCSERSETMULTI 0x545B /* Set multiport config */

/* Used for packet mode */
#define LX_TIOCPKT_DATA		 0
#define LX_TIOCPKT_FLUSHREAD	 1
#define LX_TIOCPKT_FLUSHWRITE	 2
#define LX_TIOCPKT_STOP		 4
#define LX_TIOCPKT_START	 8
#define LX_TIOCPKT_NOSTOP	16
#define LX_TIOCPKT_DOSTOP	32

struct lx_winsize {
	unsigned short ws_row;
	unsigned short ws_col;
	unsigned short ws_xpixel;
	unsigned short ws_ypixel;
};

#define LX_NCC 8
struct lx_termio {
	unsigned short c_iflag;		/* input mode flags */
	unsigned short c_oflag;		/* output mode flags */
	unsigned short c_cflag;		/* control mode flags */
	unsigned short c_lflag;		/* local mode flags */
	unsigned char c_line;		/* line discipline */
	unsigned char c_cc[LX_NCC];	/* control characters */
};

#define LX_NCCS 19
struct lx_termios {
	tcflag_t c_iflag;		/* input mode flags */
	tcflag_t c_oflag;		/* output mode flags */
	tcflag_t c_cflag;		/* control mode flags */
	tcflag_t c_lflag;		/* local mode flags */
	cc_t c_line;			/* line discipline */
	cc_t c_cc[LX_NCCS];		/* control characters */
};

/* c_cc characters */
#define LX_VINTR 0
#define LX_VQUIT 1
#define LX_VERASE 2
#define LX_VKILL 3
#define LX_VEOF 4
#define LX_VTIME 5
#define LX_VMIN 6
#define LX_VSWTC 7
#define LX_VSTART 8
#define LX_VSTOP 9
#define LX_VSUSP 10
#define LX_VEOL 11
#define LX_VREPRINT 12
#define LX_VDISCARD 13
#define LX_VWERASE 14
#define LX_VLNEXT 15
#define LX_VEOL2 16

/* c_iflag bits */
#define LX_IGNBRK	0000001
#define LX_BRKINT	0000002
#define LX_IGNPAR	0000004
#define LX_PARMRK	0000010
#define LX_INPCK	0000020
#define LX_ISTRIP	0000040
#define LX_INLCR	0000100
#define LX_IGNCR	0000200
#define LX_ICRNL	0000400
#define LX_IUCLC	0001000
#define LX_IXON	0002000
#define LX_IXANY	0004000
#define LX_IXOFF	0010000
#define LX_IMAXBEL	0020000

/* c_oflag bits */
#define LX_OPOST	0000001
#define LX_OLCUC	0000002
#define LX_ONLCR	0000004
#define LX_OCRNL	0000010
#define LX_ONOCR	0000020
#define LX_ONLRET	0000040
#define LX_OFILL	0000100
#define LX_OFDEL	0000200
#define LX_NLDLY	0000400
#define LX_NL0		0000000
#define LX_NL1		0000400
#define LX_CRDLY	0003000
#define LX_CR0		0000000
#define LX_CR1		0001000
#define LX_CR2		0002000
#define LX_CR3		0003000
#define LX_TABDLY	0014000
#define LX_TAB0		0000000
#define LX_TAB1		0004000
#define LX_TAB2		0010000
#define LX_TAB3		0014000
#define LX_XTABS	0014000
#define LX_BSDLY	0020000
#define LX_BS0		0000000
#define LX_BS1		0020000
#define LX_VTDLY	0040000
#define LX_VT0		0000000
#define LX_VT1		0040000
#define LX_FFDLY	0100000
#define LX_FF0		0000000
#define LX_FF1		0100000

/* c_cflag bit meaning */
#define LX_CBAUD	0010017
#define LX_B0		0000000		/* hang up */
#define LX_B50		0000001
#define LX_B75		0000002
#define LX_B110		0000003
#define LX_B134		0000004
#define LX_B150		0000005
#define LX_B200		0000006
#define LX_B300		0000007
#define LX_B600		0000010
#define LX_B1200	0000011
#define LX_B1800	0000012
#define LX_B2400	0000013
#define LX_B4800	0000014
#define LX_B9600	0000015
#define LX_B19200	0000016
#define LX_B38400	0000017
#define LX_EXTA		B19200
#define LX_EXTB		B38400
#define LX_CSIZE	0000060
#define LX_CS5		0000000
#define LX_CS6		0000020
#define LX_CS7		0000040
#define LX_CS8		0000060
#define LX_CSTOPB	0000100
#define LX_CREAD	0000200
#define LX_PARENB	0000400
#define LX_PARODD	0001000
#define LX_HUPCL	0002000
#define LX_CLOCAL	0004000
#define LX_CBAUDEX	0010000
#define LX_B57600 	0010001
#define LX_B115200	0010002
#define LX_B230400	0010003
#define LX_CIBAUD	002003600000	/* input baud rate (not used) */
#define LX_CRTSCTS	020000000000		/* flow control */

/* c_lflag bits */
#define LX_ISIG		0000001
#define LX_ICANON	0000002
#define LX_XCASE	0000004
#define LX_ECHO		0000010
#define LX_ECHOE	0000020
#define LX_ECHOK	0000040
#define LX_ECHONL	0000100
#define LX_NOFLSH	0000200
#define LX_TOSTOP	0000400
#define LX_ECHOCTL	0001000
#define LX_ECHOPRT	0002000
#define LX_ECHOKE	0004000
#define LX_FLUSHO	0010000
#define LX_PENDIN	0040000
#define LX_IEXTEN	0100000

/* modem lines */
#define LX_TIOCM_LE	0x001
#define LX_TIOCM_DTR	0x002
#define LX_TIOCM_RTS	0x004
#define LX_TIOCM_ST	0x008
#define LX_TIOCM_SR	0x010
#define LX_TIOCM_CTS	0x020
#define LX_TIOCM_CAR	0x040
#define LX_TIOCM_RNG	0x080
#define LX_TIOCM_DSR	0x100
#define LX_TIOCM_CD	TIOCM_CAR
#define LX_TIOCM_RI	TIOCM_RNG

/* ioctl (fd, TIOCSERGETLSR, &result) where result may be as below */
#define LX_TIOCSER_TEMT    0x01	/* Transmitter physically empty */


/* tcflow() and TCXONC use these */
#define	LX_TCOOFF	0
#define	LX_TCOON	1
#define	LX_TCIOFF	2
#define	LX_TCION	3

/* tcflush() and TCFLSH use these */
#define	LX_TCIFLUSH	0
#define	LX_TCOFLUSH	1
#define	LX_TCIOFLUSH	2

/* tcsetattr uses these */
#define	LX_TCSANOW	0
#define	LX_TCSADRAIN	1
#define	LX_TCSAFLUSH	2

/* line disciplines */
#define LX_N_TTY	0
#define LX_N_SLIP	1
#define LX_N_MOUSE	2
#define LX_N_PPP	3

#endif
