/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<sys/termio.h>
#include	<termios.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#if defined(UNIXWARE) || defined(GEMINI)
#include	<sys/filio.h>
#include	<sys/sockio.h>
#endif
#include	<sys/socket.h>
#if defined(UNIXWARE) || defined(GEMINI)
#include	<net/if.h>
#else
#include	<sys/net/if.h>
#endif
#include	"lxrun.h"
#include	"linux/socket.h"
#include	"linux/termios.h"
#include	"devmap.h"

#ifndef TCGETS
# define TCGETS		XCGETA
# define TCSETS		XCSETA
# define TCSETSW	XCSETAW
# define TCSETSF	XCSETAF
#endif

/*
 * ioctl() related system calls
 */

/*
 *	ioctl 		(PARTLY IMPLEMENTED)
 */


int lx_ioctl_default();
int lx_ioctl_audio();


/*
** Initialize the device-to-ioctl-handler map
*/
void
devmap_init(void)
{
	int i;

	for(i=0; i<NMAJ; i++)
		devmap_add_ioctl_handler(lx_ioctl_default, i);


	/* *** PUT REGISTRATION FOR NEW IOCTL HANDLERS HERE *** */

	devmap_add_ioctl_handler(lx_ioctl_audio, devmap_major("/dev/dsp"));
}



int
lx_ioctl_default(int fd, int lx_cmd, unsigned long lx_arg, int maj, int min)
{
	int		cmd;
	int		argtype;

	switch (lx_cmd)
	{
		case LX_TCGETS: 
		{
			int rv ;
			struct termios tio ;
			struct lx_termios* lx_tio=(struct lx_termios *)lx_arg;

			if(!verify_write(lx_tio, sizeof(*lx_tio)))
				/* verify_write() set errno to EFAULT */
				return -1;

			rv = ioctl(fd, TCGETS, &tio);
			if (rv) return rv ;
			memset(lx_tio, 0, sizeof(struct lx_termios));

			/* This isn't really safe, since each side has some
			   flags the other doesn't.  But the cost of doing
			   a bitwise test for each one it just too horrible
			   for words. */
			lx_tio->c_iflag = tio.c_iflag ;
			lx_tio->c_oflag = tio.c_oflag ;
			lx_tio->c_cflag = tio.c_cflag ;
			lx_tio->c_lflag = tio.c_lflag ;

			if (tio.c_lflag & ICANON)
			{
				lx_tio->c_cc[LX_VEOF]	= tio.c_cc[VEOF] ;
				lx_tio->c_cc[LX_VEOL]	= tio.c_cc[VEOL] ;
			}
			else 
			{
				lx_tio->c_cc[LX_VMIN]	= tio.c_cc[VMIN] ;
				lx_tio->c_cc[LX_VTIME]	= tio.c_cc[VTIME] ;
			}

			lx_tio->c_cc[LX_VINTR] 	= tio.c_cc[VINTR] ;
			lx_tio->c_cc[LX_VQUIT] 	= tio.c_cc[VQUIT] ;
			lx_tio->c_cc[LX_VERASE]	= tio.c_cc[VERASE] ;
			lx_tio->c_cc[LX_VKILL]	= tio.c_cc[VKILL] ;
			lx_tio->c_cc[LX_VSWTC]	= tio.c_cc[VSWTCH] ;
			lx_tio->c_cc[LX_VSTART]	= tio.c_cc[VSTART] ;
			lx_tio->c_cc[LX_VSTOP]	= tio.c_cc[VSTOP] ;
			lx_tio->c_cc[LX_VSUSP]	= tio.c_cc[VSUSP] ;
			lx_tio->c_cc[LX_VEOL]	= tio.c_cc[VEOL] ;
			return 0 ;
		}

		case LX_TCSETS:
		case LX_TCSETSW:
		case LX_TCSETSF:
		{
			struct termios tio ;
			struct lx_termios* lx_tio=(struct lx_termios *)lx_arg;
			long sco_cmd = -1;

			if (!verify_read(lx_tio, sizeof(*lx_tio)))
				/* verify_read() set errno to EFAULT */
				return -1;

			switch (lx_cmd) 
			{
				case LX_TCSETS:  sco_cmd = TCSETS;	break;
				case LX_TCSETSW: sco_cmd = TCSETSW;	break;
				case LX_TCSETSF: sco_cmd = TCSETSF;	break;
			}

			memset(&tio, 0, sizeof(struct termios)); 

			/* This isn't really safe, since each side has some
			   flags the other doesn't.  But the cost of doing
			   a bitwise test for each one it just too horrible
			   for words. */
			tio.c_iflag = lx_tio->c_iflag ;
			tio.c_oflag = lx_tio->c_oflag ;
			tio.c_cflag = lx_tio->c_cflag ;
			tio.c_lflag = lx_tio->c_lflag ;


			/* On the SCO side, these overlap (sigh) */

			if (lx_tio->c_lflag & LX_ICANON) 
			{
				tio.c_cc[VEOF]	= lx_tio->c_cc[LX_VEOF] ;
				tio.c_cc[VEOL]	= lx_tio->c_cc[LX_VEOL] ;
			}
			else
			{
				tio.c_cc[VTIME]	= lx_tio->c_cc[LX_VTIME] ;
				tio.c_cc[VMIN]	= lx_tio->c_cc[LX_VMIN] ;
			}

			tio.c_cc[VINTR]  = lx_tio->c_cc[LX_VINTR] ;
			tio.c_cc[VQUIT]  = lx_tio->c_cc[LX_VQUIT] ;
			tio.c_cc[VERASE] = lx_tio->c_cc[LX_VERASE] ;
			tio.c_cc[VKILL]	 = lx_tio->c_cc[LX_VKILL] ;
			tio.c_cc[VSWTCH] = lx_tio->c_cc[LX_VSWTC] ;
			tio.c_cc[VSTART] = lx_tio->c_cc[LX_VSTART] ;
			tio.c_cc[VSTOP]  = lx_tio->c_cc[LX_VSTOP] ;
			tio.c_cc[VSUSP]  = lx_tio->c_cc[LX_VSUSP] ;

			return ioctl(fd, sco_cmd, &tio);
		}

		case LX_TCGETA:
		       {
		       int rv ;
		       struct termio tio ;
		       struct lx_termio* lx_tio = (struct lx_termio *) lx_arg ;
		       rv = ioctl(fd, TCGETA, &tio);
		       if (rv) return rv ;

		       if (! verify_write(lx_tio, sizeof(*lx_tio)))
			       return -1; /* verify_write() set errno to EFAULT

		       memset(lx_tio, 0, sizeof(struct lx_termio));

		       /* This isn't really safe, since each side has some
			  flags the other doesn't.  But the cost of doing
			  a bitwise test for each one it just too horrible
			  for words. */
		       lx_tio->c_iflag = tio.c_iflag ;
		       lx_tio->c_oflag = tio.c_oflag ;
		       lx_tio->c_cflag = tio.c_cflag ;
		       lx_tio->c_lflag = tio.c_lflag ;


		       if (tio.c_lflag & ICANON)
			       {
			       lx_tio->c_cc[LX_VEOF]   = tio.c_cc[VEOF] ;
			       lx_tio->c_cc[LX_VEOL]   = tio.c_cc[VEOL] ;
			       }
		       else
			       {
			       lx_tio->c_cc[LX_VMIN]   = tio.c_cc[VMIN] ;
			       lx_tio->c_cc[LX_VTIME]  = tio.c_cc[VTIME] ;
			       }
		       lx_tio->c_cc[LX_VINTR]  = tio.c_cc[VINTR] ;
		       lx_tio->c_cc[LX_VQUIT]  = tio.c_cc[VQUIT] ;
		       lx_tio->c_cc[LX_VERASE] = tio.c_cc[VERASE] ;
		       lx_tio->c_cc[LX_VKILL]  = tio.c_cc[VKILL] ;
		       lx_tio->c_cc[LX_VSWTC]  = tio.c_cc[VSWTCH] ;
		       lx_tio->c_cc[LX_VSTART] = tio.c_cc[VSTART] ;
		       lx_tio->c_cc[LX_VSTOP]  = tio.c_cc[VSTOP] ;
		       lx_tio->c_cc[LX_VSUSP]  = tio.c_cc[VSUSP] ;
		       lx_tio->c_cc[LX_VEOL]   = tio.c_cc[VEOL] ;
		       return 0 ;
		       }

		case LX_TCSETA:
		case LX_TCSETAW:
		case LX_TCSETAF:
		       {
		       struct termio tio ;
		       struct lx_termio* lx_tio = (struct lx_termio *) lx_arg ;
		       long sco_cmd = -1 ;

		       if (! verify_read(lx_tio, sizeof(*lx_tio)))
			       return -1;

		       switch (lx_cmd)
			       {
			       case LX_TCSETA:  sco_cmd = TCSETA ;  break;
			       case LX_TCSETAW: sco_cmd = TCSETAW ; break;
			       case LX_TCSETAF: sco_cmd = TCSETAF ; break;
			       }

		       memset(&tio, 0, sizeof(struct termio));

		       /* This isn't really safe, since each side has some
			  flags the other doesn't.  But the cost of doing
			  a bitwise test for each one it just too horrible
			  for words. */
		       tio.c_iflag = lx_tio->c_iflag ;
		       tio.c_oflag = lx_tio->c_oflag ;
		       tio.c_cflag = lx_tio->c_cflag ;
		       tio.c_lflag = lx_tio->c_lflag ;


		       /* On the SCO side, these overlap (sigh) */

		       if (lx_tio->c_lflag & LX_ICANON)
			       {
			       tio.c_cc[VEOF]  = lx_tio->c_cc[LX_VEOF] ;
			       tio.c_cc[VEOL]  = lx_tio->c_cc[LX_VEOL] ;
			       }
		       else
			       {
			       tio.c_cc[VTIME] = lx_tio->c_cc[LX_VTIME] ;
			       tio.c_cc[VMIN]  = lx_tio->c_cc[LX_VMIN] ;
			       }

		       tio.c_cc[VINTR]	= lx_tio->c_cc[LX_VINTR] ;
		       tio.c_cc[VQUIT]	= lx_tio->c_cc[LX_VQUIT] ;
		       tio.c_cc[VERASE]	= lx_tio->c_cc[LX_VERASE] ;
		       tio.c_cc[VKILL]	= lx_tio->c_cc[LX_VKILL] ;
		       tio.c_cc[VSWTCH]	= lx_tio->c_cc[LX_VSWTC] ;
		       tio.c_cc[VSTART]	= lx_tio->c_cc[LX_VSTART] ;
		       tio.c_cc[VSTOP]	= lx_tio->c_cc[LX_VSTOP] ;
		       tio.c_cc[VSUSP]	= lx_tio->c_cc[LX_VSUSP] ;

		       return ioctl(fd, sco_cmd, &tio);
		       }

		case LX_TCSBRK:
		case LX_TCXONC:
		case LX_TCFLSH:
		case LX_TIOCEXCL:
		case LX_TIOCNXCL:
			break;

		case LX_TIOCGPGRP:	/* S.G. */
			{
			pid_t pgrp=tcgetpgrp(fd);
			if(pgrp == -1)
				return -1;
			*((pid_t *)lx_arg) = pgrp;
			return 0;
			}

		case LX_TIOCSPGRP:	/* S.G. */
			return tcsetpgrp(fd, *((pid_t *)lx_arg));

		case LX_TIOCGWINSZ:
			return ioctl(fd, TIOCGWINSZ, lx_arg);

		case LX_TIOCSWINSZ:
			break;

		case LX_FIONREAD:
			return ioctl(fd, FIONREAD, lx_arg);
			
		case LX_FIONBIO:
			return ioctl(fd, FIONBIO, lx_arg);

		       /* It's a gift that the structures seem to
			  match "well enough" */
		case LX_SIOCGIFCONF:
			return ioctl(fd, SIOCGIFCONF, lx_arg);

		case LX_SIOCGIFFLAGS:
			return ioctl(fd, SIOCGIFFLAGS, lx_arg);
 
		case LX_SIOCGIFADDR:
			return ioctl(fd, SIOCGIFADDR, lx_arg);
 
		case LX_SIOCGIFNETMASK:
			return ioctl(fd, SIOCGIFNETMASK, lx_arg);
 
		case LX_SIOCGIFBRDADDR:
			return ioctl(fd, SIOCGIFBRDADDR, lx_arg);

		case LX_TIOCSETD:
		case LX_TIOCGETD:
		case LX_FIONCLEX:
		case LX_FIOCLEX:
		case LX_FIOASYNC:
		case LX_TIOCOUTQ:
		case LX_TIOCSTI:
		case LX_TIOCSCTTY:
		case LX_TIOCMGET:
		case LX_TIOCMBIS:
		case LX_TIOCMBIC:
		case LX_TIOCMSET:
		case LX_TIOCGSOFTCAR:
		case LX_TIOCSSOFTCAR:
		case LX_TIOCLINUX:
		case LX_TIOCCONS:
		case LX_TIOCGSERIAL:
		case LX_TIOCSSERIAL:
		case LX_TIOCPKT:
		case LX_TIOCNOTTY:
		case LX_TCSBRKP:
		case LX_TIOCTTYGSTRUCT:
		case LX_TIOCSERCONFIG:
		case LX_TIOCSERGWILD:
		case LX_TIOCSERSWILD:
		case LX_TIOCGLCKTRMIOS:
		case LX_TIOCSLCKTRMIOS:
		case LX_TIOCSERGSTRUCT:
		case LX_TIOCSERGETLSR:
		case LX_TIOCSERGETMULTI:
		case LX_TIOCSERSETMULTI:
			break;
			
	}
	errno = ENOSYS;
	return -1;
}



/*
** ioctl mapping for OSS audio drivers
*/
int
lx_ioctl_audio(int fd, int lx_cmd, unsigned long lx_arg, int maj, int min)
{
	unsigned char	cmdkey = (lx_cmd >> 8) & 0xff;

	if(cmdkey == 'P' ||	/* DSP */
	   cmdkey == 'M' ||	/* Mixer */
	   cmdkey == 'm' ||	/* MIDI */
	   cmdkey == 'C' ||	/* Co-Processor */
	   cmdkey == 'T' ||	/* Timer */
	   cmdkey == 'Q')	/* Sequencer */
		{
		/*
		** Since SCO audio is essentially the same as
		** Linux audio, there's not much conversion to
		** do.  (There wouldn't be any, but the direction
		** information is handled differently.)
		*/
		unsigned int sco_cmd, dirbits;

		/*
		** TTD: This should be changed so we use macros from
		** ioctl.h and/or soundcard.h instead of using hard-
		** coded values.
		*/

		/* Collect direction bits from original command */
		dirbits = lx_cmd & 0xc0000000;

		/* Perform transformation */
		if(dirbits & 0x80000000)
			{
			dirbits &= ~0x80000000;
			dirbits |=  0x20000000;
			}

		/* Mask them back in */
		sco_cmd = (lx_cmd & 0x1ffffff) | dirbits;

		/* Call the audio ioctl */
		return ioctl(fd, sco_cmd, lx_arg);
		}

	errno=ENOSYS;
	return -1;
}

