/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<sys/types.h>
#include	<sys/socket.h>
#include	<unistd.h>
#include	"lxrun.h"
#include	"linux/ipc.h"

extern int lx_pipe();
extern int lx_semop();
extern int lx_semget();
extern int lx_semctl();
extern int lx_msgsnd();
extern int lx_msgrcv();
extern int lx_msgget();
extern int lx_msgctl();
extern int lx_shmat();
extern int lx_shmdt();
extern int lx_shmget();
extern int lx_shmctl();


/*
** Since Linux pipes can be bi-directional, use a socketpair to
** implement a pipe.
*/
int
lx_pipe(int filedes[2])
{
#ifndef UNIDIRECTIONAL_PIPES
	return socketpair(AF_UNIX, SOCK_STREAM, 0, filedes);
#else
	return pipe(filedes);
#endif
}


int
lx_ipc(unsigned call, int arg1, int arg2, int arg3, void *arg4)
{
	int	version	= call >> 16;

	/*
	 * Linux binaries should all be version 0
	 * version 1 is only used by the iBCS2 emulatior
	 */
	if (version != 0)
	{
		errno = ENOSYS;
		return -1;
	}

	switch (call & 0xffff)
	{
		case LX_SEMOP:
			return lx_semop(arg1, arg4, arg2);

		case LX_SEMGET:
			return lx_semget(arg1, arg2, arg3);

		case LX_SEMCTL: 
			if (!arg4)
			{
				errno = EINVAL;
				return -1;
			}
			return lx_semctl(arg1, arg2, arg3, *(long *)arg4);

		case LX_MSGSND:
			return lx_msgsnd(arg1, arg4, arg2, arg3);

		case LX_MSGRCV:
		{
			struct lx_ipc_kludge tmp;

			if (!arg4)
			{
				errno = EINVAL;
				return -1;
			}

			tmp	= *(struct lx_ipc_kludge *)arg4;
			return lx_msgrcv(arg1, tmp.ptr, arg2, tmp.type, arg3);
		}

		case LX_MSGGET:
			return lx_msgget(arg1, arg2);

		case LX_MSGCTL:
			return lx_msgctl(arg1, arg2, arg4);

		case LX_SHMAT:
			return lx_shmat(arg1, arg4, arg2, arg3);

		case LX_SHMDT: 
			return lx_shmdt(arg4);

		case LX_SHMGET:
			return lx_shmget(arg1, arg2, arg3);

		case LX_SHMCTL:
			return lx_shmctl(arg1, arg2, arg4);
	}

	errno	= EINVAL;
	return -1;
}
