/*
 * Copyright (C) 1998 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<unistd.h>
#include	<sys/socket.h>
#include	<netdb.h>
#include	"lxrun.h"
#include	"auxv.h"
#include	"linux/elf.h"
#include	"version.h"
#include	"devmap.h"

char	*linux_root	= NULL;
char	*linux_uname	= NULL;
char	emulator[PATH_MAX];	/* Contains full pathname of invoked lxrun */

/*
 * LINUX emulator for OpenServer 5.0 / UnixWare
 */

int
main(int argc, char **argv, char **envp)
{
	char	*display, hostname[MAXHOSTNAMELEN+1];
	char	displaybuf[MAXHOSTNAMELEN+20];
	char	**pp;
	auxv_t	*auxv;
	auxv_t	*a;
	int	i;
	Elf32_Phdr	*phdr;
	int		phnum;
	int		phent;

	/*
	 * find the aux vector
	 */
	pp = envp;
	while (*pp)
		++pp;
	auxv = (auxv_t *) (pp + 1);

	for (a = auxv; a->a_type != AT_NULL; a++) {
		switch (a->a_type) {
			case AT_PHDR:
				phdr = a->a_un.a_ptr;
				break;
			case AT_PHENT:
				phent = a->a_un.a_val;
				break;
			case AT_PHNUM:
				phnum = a->a_un.a_val;
				break;
			default:
				/*
				 * UnixWare puts some additional entries
				 * in the aux vector which conflict with
				 * the Linux dynamic linker so we just
				 * ignore them
				 */
				if (a->a_type > AT_ENTRY)
					a->a_type = AT_IGNORE;
				break;
		}
	}

	for (i = 0; i < phnum; i++) {
		extern caddr_t	minimum_brk;
		extern caddr_t	current_brk;

		if (phdr[i].p_type == PT_LOAD) {
			current_brk =
			minimum_brk = (caddr_t)(phdr[i].p_vaddr + phdr[i].p_memsz);
		}
	}

	linux_root = getenv("LINUX_ROOT");
	linux_uname = getenv("LINUX_UNAME");

#ifdef DEF_LINUX_ROOT
	if (! linux_root)
		linux_root = DEF_LINUX_ROOT;
#endif

#ifndef NO_DISPLAY_HACK
	/*
	** I have a feeling this hack is only useful on OpenServer
	** machines.  Is this true?  If so, does someone want to
	** figure out a UW/Gemini equivalent?
	*/
	if ((display = getenv("DISPLAY")) != NULL && display[0] == ':') {
		if (gethostname(hostname, sizeof(hostname))) {
			perror("gethostname");
			exit(1);
		}

		snprintf(displaybuf, sizeof(displaybuf), "DISPLAY=%s%s",
					hostname, display);

		if (putenv(displaybuf)) {
			fprintf(stderr, "putenv(%s) failed\n", displaybuf);
			exit(1);
		}
	}
#endif

	if (init_signals() != 0)
		fatal(errno, "can't set up signal handlers");

	devmap_init();		/* S.G. */

	run_elf_interpreter("/lib/ld-linux.so.1", argv, envp, auxv);

	fatal(errno, "can't load /lib/ld-linux.so.1");

	return 1;
}
