/*
 * Copyright (C) 1996-1997 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<stdio.h>
#include	<stdlib.h>
#include	<assert.h>
#include	<errno.h>
#include	<string.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<sys/mman.h>
#include	"lxrun.h"
#include	"auxv.h"
#include	"linux/a.out.h"
#include	"linux/elf.h"
#ifdef ELF_DEBUG
#include	<stdio.h>
#endif


caddr_t		current_brk = 0;
caddr_t		minimum_brk = 0;


struct aout_exec_info
{
	ushort	magic;
	off_t	text_off;
	caddr_t	text_addr;
	size_t	text_size;
	off_t	data_off;
	caddr_t	data_addr;
	size_t	data_size;
	caddr_t	bss_addr;
	size_t	bss_size;
	caddr_t	start_addr;
};


static int
get_aout_header(int fd, struct aout_exec_info *info)
{
	long		filesz;
	struct lx_exec	ex;

	filesz = lseek(fd, 0L, SEEK_END);
	if (lseek(fd, 0L, SEEK_SET) != 0 ||
		read(fd, &ex, sizeof(ex)) != sizeof(ex))
	{
		errno = EACCES;
		return -1;
	}

	switch (ex.a_magic)
	{
		case LX_OMAGIC:
			info->magic	= ex.a_magic;
			info->text_off	= sizeof(struct lx_exec);
			info->text_addr	= 0;
			info->text_size	= ex.a_text;
			info->data_off	= info->text_off + info->text_size;
			info->data_addr	= info->text_addr + info->text_size;
			info->data_size	= ex.a_data;
			info->bss_addr	= info->data_addr + info->data_size;
			info->bss_size	= ex.a_bss;
			info->start_addr= (caddr_t)ex.a_entry;
			break;

		case LX_ZMAGIC:
			info->magic	= ex.a_magic;
			info->text_off	= BLOCK_SIZE;
			info->text_addr	= 0;
			info->text_size	= ex.a_text;
			info->data_off	= info->text_off + info->text_size;
			info->data_addr	= info->text_addr + BLOCK_ALIGN(info->text_size);
			info->data_size	= ex.a_data;
			info->bss_addr	= info->data_addr + info->data_size;
			info->bss_size	= ex.a_bss;
			info->start_addr= (caddr_t)ex.a_entry;
			break;

		case LX_QMAGIC:
			info->magic	= ex.a_magic;
			info->text_off	= 0;
			info->text_addr	= (caddr_t)PAGE_SIZE;
			info->text_size	= ex.a_text;
			info->data_off	= info->text_off + info->text_size;
			info->data_addr	= info->text_addr + BLOCK_ALIGN(info->text_size);
			info->data_size	= ex.a_data;
			info->bss_addr	= info->data_addr + info->data_size;
			info->bss_size	= ex.a_bss;
			info->start_addr= (caddr_t)ex.a_entry;
			break;

		default:
			errno = ENOEXEC;
			return -1;
	}

	if (filesz < (info->data_off + info->data_size + ex.a_syms))
	{
		errno = ENOEXEC;
		return -1;
	}

	if (ex.a_trsize || ex.a_drsize)
	{
		errno = ENOEXEC;
		return -1;
	}

	return 0;
}


static int
map_aout_file(int fd, struct aout_exec_info *info)
{
	int	r	= 0;
	int	z;
	caddr_t	text, data, bss;

	if ((z = open("/dev/zero", O_RDWR)) < 0)
	{
		fatal(errno, "can't open /dev/zero");
		return -1;
	}

/*
 * unfortunately ZMAGIC binaries start at offset 1024 into the
 * executable file which means that we can't map them directly
 * using mmap(), so we treat them like impure OMAGIC binaries
 * and read them into memory
 */
	if (info->magic == LX_OMAGIC || info->magic == LX_ZMAGIC)
	{
		size_t	n	= info->text_size + info->data_size;

		text = mmap(info->text_addr, n,
			PROT_READ|PROT_WRITE|PROT_EXEC,
			MAP_FIXED|MAP_PRIVATE, z, 0);

		if (text != info->text_addr)
			fatal(errno, "can't map memory for program");

		if (lseek(fd, info->text_off, 0) != info->text_off
			|| read(fd, text, n) != n)
			r = -1;
	}
	else if (info->magic == LX_QMAGIC)
	{
		text = mmap(info->text_addr, info->text_size,
			PROT_READ|PROT_EXEC,
			MAP_FIXED|MAP_PRIVATE, fd, info->text_off);

		if (text != info->text_addr)
			fatal(errno, "can't map text");

		data = mmap(info->data_addr, info->data_size,
			PROT_READ|PROT_WRITE|PROT_EXEC,
			MAP_FIXED|MAP_PRIVATE, fd, info->data_off);

		if (data != info->data_addr)
			fatal(errno, "can't map data");
	}
	else
	{
		errno = ENOEXEC;
	}

	if (r == 0 && info->bss_size)
	{
		caddr_t		bss;
		unsigned long	bss_start, bss_end;

		bss_start = (unsigned long)info->bss_addr;
		bss_end   = bss_start + info->bss_size;
		bss_start = PAGE_ALIGN(bss_start);
		bss_end   = PAGE_ALIGN(bss_end);
		if (bss_end > bss_start)
		{
			bss = mmap((caddr_t)bss_start, bss_end - bss_start,
				PROT_READ|PROT_WRITE|PROT_EXEC,
				MAP_FIXED|MAP_PRIVATE, z, 0);

			if (bss != (caddr_t)bss_start)
				fatal(errno, "can't map bss");
		}
	}

	close(z);

	return r;
}


static int
run_aout_program(const char *name, int fd, char **argv, char **envp)
{
	struct aout_exec_info	e;

	if (get_aout_header(fd, &e) < 0)
		return -1;

	if (map_aout_file(fd, &e) < 0)
		return  -1;

	close(fd);

	minimum_brk	= e.text_addr + e.text_size;
	current_brk	= e.bss_addr  + e.bss_size;

	lxstart((void*)e.start_addr, argv, envp, NULL);

	/* lxstart() does not return */
	return -1;
}


/*
 * ELF support
 */

struct elf_exec_info
{
	Elf32_Ehdr	elf_hdr;
	Elf32_Addr	load_addr;
	Elf32_Addr	end_addr;
	Elf32_Phdr	*program_header;
	char		*interpreter;
	Elf32_Dyn	*dynamic;
};

static int
get_elf_header(int fd, struct elf_exec_info *info)
{
	Elf32_Ehdr	*eh	= &info->elf_hdr;

	info->load_addr		= 0;
	info->program_header	= 0;
	info->interpreter	= 0;
	info->dynamic		= 0;

	if (lseek(fd, 0L, SEEK_SET) != 0 ||
		read(fd, eh, sizeof(*eh)) != sizeof(*eh))
	{
		errno = EACCES;
		return -1;
	}

	if ( memcmp(eh->e_ident, ELFMAG, SELFMAG) != 0 ||
		eh->e_ident[EI_CLASS] != ELFCLASS32 ||
		eh->e_ident[EI_DATA] != ELFDATA2LSB ||
		(eh->e_type != ET_EXEC && eh->e_type != ET_DYN) ||
		(eh->e_machine != EM_386 && eh->e_machine != EM_486) ) {
		errno = ENOEXEC;
		return -1;
	}

	return 0;
}

#define	MPROT(p)	( \
	( ((p)->p_flags & PF_R) ? PROT_READ  : 0 ) |	\
	( ((p)->p_flags & PF_W) ? PROT_WRITE : 0 ) |	\
	( ((p)->p_flags & PF_X) ? PROT_EXEC  : 0 ) 	\
)

#define	SAME_ALIGNMENT(p, q)	\
	( ((p)->p_vaddr - (q)->p_vaddr) == ((p)->p_offset - (q)->p_offset) )

static int
map_elf_file(int fd, struct elf_exec_info *info)
{
	Elf32_Ehdr	*eh	= &info->elf_hdr;
	int		i	= eh->e_phnum * eh->e_phentsize;
	Elf32_Phdr	*phdrs	= alloca(i);
	Elf32_Phdr	*first	= 0;
	Elf32_Phdr	*last	= 0;
	Elf32_Phdr	*p;
	Elf32_Addr	addr;
	Elf32_Word	reloc;
	off_t		offset;
	size_t		len;
	int		prot	= 0;
	int		flags	= MAP_PRIVATE;
	caddr_t		mem;
	int		phdr	= -1;
	int		interp	= -1;
	int		dyn	= -1;

	/*
	 * read the program header table and map in the loadable segments
	 */

	if (lseek(fd, eh->e_phoff, SEEK_SET) != eh->e_phoff) {
		return -1;
	}

	if (read(fd, phdrs, i) != i) {
		return -1;
	}

	for (i = 0; (unsigned)i < eh->e_phnum; i++) {

		switch (phdrs[i].p_type)
		{
			case PT_PHDR:
				phdr = i;
				break;

			case PT_INTERP:
				interp = i;
				break;

			case PT_DYNAMIC:
				dyn = i;
				break;

			case PT_LOAD:
				if (! first)
					first = &phdrs[i];
				last = &phdrs[i];
				break;
		}

	}

	/*
	 * map everything with the protection appropriate to
	 * the first loadable segment
	 */
	addr	= first->p_vaddr  & ~(PAGE_SIZE-1);
	len	= (last->p_vaddr + last->p_memsz) - addr;
	prot	= MPROT(first);
	flags	= (addr) ? (MAP_PRIVATE | MAP_FIXED) : MAP_PRIVATE;
	offset	= first->p_offset & ~(PAGE_SIZE-1);

	mem	= mmap((caddr_t)addr, len, prot, flags, fd, offset);
#ifdef ELF_DEBUG
fprintf(stderr, "mmap(0x%x, 0x%x, 0x%x, 0x%x, %d, 0x%x) returns 0x%x\n",
	addr, len, prot, flags, fd, offset, mem);
#endif

	if (mem == (caddr_t)-1) {
		return -1;
	}
	reloc		= mem - addr;
	info->load_addr	= mem;
	info->end_addr	= last->p_vaddr + last->p_memsz + reloc;

	/*
	 * fix the protections and remap the other segments if
	 * necessary
	 */
	for (p = first + 1; p <= last; p++) {
		if (p->p_type != PT_LOAD)
			continue;

		addr	= reloc + (p->p_vaddr  & ~(PAGE_SIZE-1));
		len	= p->p_memsz + (p->p_vaddr & (PAGE_SIZE-1));
		prot	= MPROT(p);

		if (SAME_ALIGNMENT(p, first)) {
			int	r;
			r = mprotect(addr, len, prot);
#ifdef ELF_DEBUG
fprintf(stderr, "mprotect(0x%x, 0x%x, 0x%x) returns %d\n",
	addr, len, prot, r);
#endif
		} else {
			flags	= MAP_PRIVATE | MAP_FIXED;
			offset	= p->p_offset & ~(PAGE_SIZE-1);
			mem	= mmap(addr, len, prot, flags, fd, offset);
#ifdef ELF_DEBUG
fprintf(stderr, "mmap(0x%x, 0x%x, 0x%x, 0x%x, %d, 0x%x) returns 0x%x\n",
	addr, len, prot, flags, fd, offset, mem);
#endif
		}

		if (prot & PROT_WRITE) {
			int	bss	= p->p_memsz - p->p_filesz;
			caddr_t m	= addr + len - bss;
			int	n;

			/*
			 * clear any residual bytes at the end of the last
			 * page of the mapping - note that mem is page aligned
			 * which makes the calculation a little simpler
			 */

			if ((n = (int)m & (PAGE_SIZE-1)) != 0) {
				n = PAGE_SIZE - n;
#ifdef ELF_DEBUG
fprintf(stderr, "clearing 0x%x bytes of BSS at address 0x%08X\n", n, m);
#endif
				memset(m, 0, n);
				m   += n;
				bss -= n;

			}

			if (bss > 0) {
				int	z;

				if ((z = open("/dev/zero", O_RDWR)) < 0)
					fatal(errno, "can't open /dev/zero");
	
				bss = (bss + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1);
				flags = MAP_PRIVATE | MAP_FIXED;
	
				if (mmap(m, bss, prot, flags, z, 0) != m)
					fatal(errno, "can't map bss");
#ifdef 	ELF_DEBUG
fprintf(stderr, "mapping additional pages of BSS\n");
fprintf(stderr, "mmap(0x%08X, 0x%x, 0x%x, 0x%x, %d, 0x%x) returned 0x%08X\n",
	m, bss, prot, flags, z, 0, m);
#endif
				close(z);
			}
		}
	}



	/*
	 * check that if we have seen PHDR, INTERP or DYN entries
	 * they are mapped somewhere within the loaded segments
	 */
	if (phdr >= 0) {
		Elf32_Addr	addr	= phdrs[phdr].p_vaddr;
		Elf32_Word	len	= phdrs[phdr].p_memsz;

		if (eh->e_type == ET_DYN)
			addr += info->load_addr;
#ifdef ELF_DEBUG
fprintf(stderr, "phdrs: addr = 0x%x, len = 0x%x\n", addr, len);
#endif
		if (verify_read((void *)addr, len))
			info->program_header = (Elf32_Phdr *)addr;
		else
			warning(0, "PHDR exists but is not mapped");
	}

	if (interp >= 0) {
		Elf32_Addr	addr	= phdrs[interp].p_vaddr;
		Elf32_Word	len	= phdrs[interp].p_memsz;

#if 1
		/*
		 * this is an ugly hack so that we can load and run
		 * standard OSR5 and UnixWare ELF - for some unknown
		 * reason the SVr4 linker sets p_vaddr for the INTERP
		 * segment to 0 even though it is in fact always mapped.
		 * Here we kludge together what p_vaddr *should* be
		 * based on where we expect that it is actually mapped
		 */
		if (addr == 0)
			addr = info->load_addr + phdrs[interp].p_offset;
		else
#endif
		if (eh->e_type == ET_DYN)
			addr += info->load_addr;
#ifdef ELF_DEBUG
fprintf(stderr, ".interp: addr = 0x%x, len = 0x%x\n", addr, len);
#endif
		if (verify_read((void *)addr, len))
			info->interpreter = (char *)addr;
		else
			warning(0, "INTERP exists but is not mapped");
	}

	if (dyn >= 0) {
		Elf32_Addr	addr	= phdrs[dyn].p_vaddr;
		Elf32_Word	len	= phdrs[dyn].p_memsz;

		if (eh->e_type == ET_DYN)
			addr += info->load_addr;
#ifdef ELF_DEBUG
fprintf(stderr, ".dynamic: addr = 0x%x, len = 0x%x\n", addr, len);
#endif

		if (verify_read((void *)addr, len))
			info->dynamic = (Elf32_Dyn *)addr;
		else
			warning(0, "DYNAMIC exists but is not mapped");
	}

	return 0;
}

#define	AUX(i, t, v)	(aux[(i)].a_type = (t), aux[(i)].a_un.a_val = (long)(v))

#define	MAX_AUX		15

int
run_elf_interpreter(const char *name, char **argv, char **envp, auxv_t *auxv)
{
	auxv_t			*a;
	int			ifd;
	struct elf_exec_info	interp;
	char			path[PATH_MAX];

	if (lx_pathname(name, path, PATH_MAX) < 0)
		return -1;
#ifdef ELF_DEBUG
fprintf(stderr, "program interpreter '%s' mapped to '%s'\n", name, path);
#endif

	if ((ifd = open(path, O_RDONLY)) < 0)
		return -1;
#ifdef ELF_DEBUG
fprintf(stderr, "open succeeded\n");
#endif


	if (get_elf_header(ifd, &interp) < 0) {
		close(ifd);
		return -1;
	}
#ifdef ELF_DEBUG
fprintf(stderr, "get_elf_header succeeded\n");
#endif

	if (interp.elf_hdr.e_type != ET_DYN) {
		close(ifd);
		errno = ENOEXEC;
		return -1;
	}

	if (map_elf_file(ifd, &interp) < 0) {
		close(ifd);
		return -1;
	}

	close(ifd);
#ifdef ELF_DEBUG
fprintf(stderr, "loaded OK, load address = 0x%08X, current brk = 0x%08X\n",
	interp.load_addr, current_brk);
#endif

	/*
	 * patch the base address of the interpreter
	 * into the AT_BASE entry in the aux vector
	 */
	for (a = auxv; a->a_type != AT_NULL; a++) {
		if (a->a_type == AT_BASE) {
			a->a_un.a_val = interp.load_addr;
			break;
		}
	}

#if 0
{
    char **p;
    
    fprintf(stderr, "environment passed to lxstart:\n");
    for (p = envp; *p; p++)
	fprintf(stderr, "%2d: '%s'\n", p - envp, *p);

    fprintf(stderr, "auxv passed to lxstart:\n");
	for (a = auxv; a->a_type != AT_NULL; a++) {
		fprintf(stderr, "type = %2d, value = %8x\n", a->a_type,
			a->a_un.a_val);
	}
}
#endif
	lxstart((void*) (interp.elf_hdr.e_entry + interp.load_addr),
		argv, envp, auxv);

	return -1;
}

static int
run_elf_program(const char *name, int fd, char **argv, char **envp)
{
	auxv_t			aux[MAX_AUX];
	struct elf_exec_info	prog;

#ifdef ELF_DEBUG
fprintf(stderr, "trying to load '%s'\n", name);
#endif

	if (get_elf_header(fd, &prog) < 0)
		return -1;

	if (prog.elf_hdr.e_type != ET_EXEC) {
		errno = ENOEXEC;
		return -1;
	}

	if (map_elf_file(fd, &prog) < 0)
		return -1;

	close(fd);

	minimum_brk	= 
	current_brk	= prog.end_addr;

#ifdef ELF_DEBUG
fprintf(stderr, "loaded OK, load address = 0x%08X, current brk = 0x%08X\n",
	prog.load_addr, current_brk);
#endif

	if (prog.interpreter) {
		/*
		 * build the aux vector
		 */
		AUX(0, AT_PHDR,   prog.program_header);
		AUX(1, AT_PHENT,  prog.elf_hdr.e_phentsize);
		AUX(2, AT_PHNUM,  prog.elf_hdr.e_phnum);
		AUX(3, AT_PAGESZ, PAGE_SIZE);
		AUX(4, AT_BASE,   0);
		AUX(5, AT_FLAGS,  0);
		AUX(6, AT_ENTRY,  prog.elf_hdr.e_entry);
		AUX(7, AT_NULL,   0);

		run_elf_interpreter(prog.interpreter, argv, envp, aux);

	} else {
		/*
		 * static ELF doesn't need an aux vector, but lxstart()
		 * uses the presence of an aux vector to distinguish
		 * between COFF and ELF which require slightly different
		 * stack layouts at start-up, so we supply a dummy aux
		 * vector even for static linked ELF
		 */
		AUX(0, AT_NULL,   0);
		lxstart((void*) prog.elf_hdr.e_entry, argv, envp, aux);
	}

	/*NOTREACHED*/

	return -1;
}

#define	LINUX_EXEC_AOUT		1
#define	LINUX_EXEC_ELF		2

int
linux_exec_type(int fd)
{
	union {
		struct lx_exec	a;
		Elf32_Ehdr	e;
	} hdr;

	if (read(fd, &hdr, sizeof(hdr)) != sizeof(hdr))
		return -1;

	if (hdr.a.a_magic == LX_OMAGIC || hdr.a.a_magic == LX_ZMAGIC
		|| hdr.a.a_magic == LX_QMAGIC) {
		return LINUX_EXEC_AOUT;
	}

#ifdef ELFMARK_HACK
	if (hdr.e.e_flags == *(unsigned long *)"LXRN")
		return LINUX_EXEC_ELF;
#endif

	/* We consider a Linux ELF binary to be anything that has
		an interpreter name that contains "linux".   Any
		failure in this logic returns "uknown" which will
		likely result in a handoff to the native exec() */

	if (memcmp(hdr.e.e_ident, ELFMAG, SELFMAG) == 0) {
		Elf32_Ehdr *eh = &hdr.e ;
		int phsize = eh->e_phentsize * eh->e_phnum;
		Elf32_Phdr *phdr;
		Elf32_Word i;
 
		if (!(phdr = alloca(phsize)))
			fatal(0, "Cannot alloca() %d bytes", phsize);
 
		if (lseek(fd, eh->e_phoff, SEEK_SET) != eh->e_phoff)
			return -1;
 
		if (read(fd, phdr, phsize) < phsize)
			return -1;
 
		for (i = 0; i < eh->e_phnum; i++, phdr++) {
			if ( phdr->p_type == PT_INTERP ) {
				char *interp_name;
				Elf32_Word sz = phdr->p_filesz;
 
				if (!(interp_name = alloca(sz)))
					return -1;
 
				if (read(fd, interp_name, sz) < sz)
					return -1;
 
#ifdef ELF_DEBUG
 fprintf(stderr, "Interpreter section found '%s'\n", interp_name);
 fprintf(stderr, "phdr %d size %d offset %d\n",
			phdr->p_type, sz, phdr->p_offset);
#endif

				if (!strstr(interp_name, "linux"))
					return -1;
 
				}
			}
 
		return LINUX_EXEC_ELF;
	}

	return -1;
}

int
is_linux_exec(const char *filename)
{
	int	fd;
	int	type;

	if ((fd = open(filename, O_RDONLY)) < 0)
		return 0;

	type = linux_exec_type(fd);

	close(fd);

	return (type == LINUX_EXEC_AOUT || type == LINUX_EXEC_ELF);
}

#ifdef LXRUN_AUTO_PATH_BEHAVIOR
char *
find_in_path(const char *progname)
{
	static char	qualified[PATH_MAX];
	char		*path, *tok;

	if(strchr(progname, '/') || ((path = getenv("PATH")) == NULL))
		return (char *)progname;

	tok = strtok(path, ":");

	for(tok = strtok(path, ":"); tok; tok = strtok(NULL, ":")) {

		if(tok[0]='\0')
			tok=".";

		snprintf(qualified, sizeof(qualified), "%s/%s", tok, progname);

		/* Should this be X_OK instead of R_OK? */
		/* Lxrun requires read permissions. */
		if(eaccess(qualified, R_OK) == 0)
			return qualified;
	}

	return (char *)progname;
}
#endif	/* LXRUN_AUTO_PATH_BEHAVIOR */

int
run_program(const char *progname, char **argv, char **envp)
{
	int	fd;
	int	err;

#ifdef LXRUN_AUTO_PATH_BEHAVIOR
	if ((fd = open(find_in_path(progname), O_RDONLY)) < 0)
#else
	if ((fd = open(progname, O_RDONLY)) < 0)
#endif
		return -1;

	switch (linux_exec_type(fd))
	{
		case LINUX_EXEC_AOUT:
			run_aout_program(progname, fd, argv, envp);
			break;

		case LINUX_EXEC_ELF:
			run_elf_program(progname, fd, argv, envp);
			break;

		default:
			errno = ENOEXEC;
			break;
	}

	err = errno;	/* save errno */

	close(fd);

	if ((errno = err) == 0)
		errno = ENOEXEC;

	return -1;
}


static void *
do_load_library(int fd)
{
	struct aout_exec_info	e;
	caddr_t			load_addr;
	unsigned long		offset;

	if (get_aout_header(fd, &e) < 0)
		return (void *) -1;

	if (e.magic != LX_ZMAGIC && e.magic != LX_QMAGIC)
	{
		errno = ENOEXEC;
		return (void *) -1;
	}

	/*
	 * calculate library load address from the entry point
	 * and update the addresses in the aout_exec_info structures
	 */
	load_addr = (caddr_t)((unsigned long) e.start_addr & ~(PAGE_SIZE-1));
	offset    = load_addr - e.text_addr;

	e.text_addr	+= offset;
	e.data_addr	+= offset;
	e.bss_addr	+= offset;


	if (map_aout_file(fd, &e) < 0)
		return (void *) -1;

	return (void *) e.start_addr;
}


void *
load_library(const char *libname)
{
	int	fd;
	void	*p;

	if ((fd = open(libname, O_RDONLY)) < 0)
		return (void *) -1;

	p = do_load_library(fd);

	close(fd);

	return p;
}
