/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#if !defined(LXRUN_H)
#define LXRUN_H

#define	BLOCK_SIZE	1024

#if !defined(PAGE_SIZE)
#define	PAGE_SIZE	4096
#endif

#ifndef PATH_MAX
#define	PATH_MAX	1024
#endif

#define BLOCK_ALIGN(x)	(((ulong_t)(x) + BLOCK_SIZE - 1) & ~(BLOCK_SIZE - 1))
#define PAGE_ALIGN(x)	(((ulong_t)(x) + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1))

extern int	run_program(const char *progname, char **argv, char **envp);
extern void	lxstart(void *start, void *argv, void *envp, void *auxv);
extern void	*load_library(const char *libname);
extern int	is_linux_exec(const char *filename);
extern int	init_signals();

extern int	lx_errno(int err);
extern int	lx_pathname(const char *lx_path, char *os_path, int len);

extern int	getbyte(const void *addr);
extern int	putbyte(void *addr, int value);

extern int	verify_read(const void *addr, unsigned len);
extern int	verify_write(void *addr, unsigned len);

extern void	fatal(int err, char *format, ...);
extern void	warning(int err, char *format, ...);

#if defined(TRACE)
extern void	tprintf(char *format, ...);
extern void	trace_stack(void *frame);
extern void	trace_syscall(int n, int *regs, int rval, int err);
extern void	trace_signal(int sig, void *info, void *context);
#endif

extern int	getdomainname(char *name, int namelen);
extern void	*alloca(unsigned);

#endif
