/*
 * Copyright (C) 1996-1997 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<unistd.h>
#include	<sys/socket.h>
#include	<netdb.h>
#include	"lxrun.h"
#include	"version.h"
#include	"devmap.h"

char	*linux_root	= NULL;
char	*linux_uname	= NULL;
char	emulator[PATH_MAX];	/* Contains full pathname of invoked lxrun */

/*
 * LINUX emulator for OpenServer 5.0 / UnixWare
 */

int
main(int argc, char **argv, char **envp)
{
	char	*display, hostname[MAXHOSTNAMELEN+1];
	char	displaybuf[MAXHOSTNAMELEN+20];
	char	*pathp = getenv("PATH");

	if(access(argv[0], X_OK) == 0)
	{
		strcpy(emulator, argv[0]);
	}
	else
	{
		while (pathp=strtok(pathp, ":"))
		{
			strcpy(emulator, pathp);
			strcat(emulator, "/");
			strcat(emulator, argv[0]);

			if(access(emulator, X_OK) == 0)
				break;

			pathp=NULL;
		}
	}
	if(access(emulator, X_OK) != 0)
	{
		fprintf(stderr, "%s: Can't find full path of lxrun binary!",
				argv[0]);
		return 1;
	}

	if (argc < 2)
	{
		fprintf(stderr, "Linux Emulator version %s, ", build_version);
		fprintf(stderr, "built on %s at %s\n", build_date, build_time);
		fprintf(stderr, "usage: %s linux_prog [args ...]\n", argv[0]);
		return 1;
	}

#if defined(TRACE)
	{
		int	n;
		if ((n = trace_init(argc, argv)) < 0)
			return 1;
		argv += n;
	}
#else
	argv += 1;
#endif

#ifdef DEF_LINUX_ROOT
	if ((linux_root = getenv("LINUX_ROOT")) == NULL)
		linux_root = DEF_LINUX_ROOT;
#else
	linux_root = getenv("LINUX_ROOT");
#endif

	linux_uname = getenv("LINUX_UNAME");

#ifndef NO_DISPLAY_HACK
	/*
	** I have a feeling this hack is only useful on OpenServer
	** machines.  Is this true?  If so, does someone want to
	** figure out a UW/Gemini equivalent?
	*/
	if ((display = getenv("DISPLAY")) != NULL && display[0] == ':') {
		if (gethostname(hostname, sizeof(hostname))) {
			perror("gethostname");
			exit(1);
		}

		snprintf(displaybuf, sizeof(displaybuf), "DISPLAY=%s%s",
					hostname, display);

		if (putenv(displaybuf)) {
			fprintf(stderr, "putenv(%s) failed\n", displaybuf);
			exit(1);
		}
	}
#endif

	if (init_signals() != 0)
		fatal(errno, "can't set up signal handlers");

	devmap_init();		/* S.G. */

	run_program(*argv, argv, envp);

	/*
	 * like exec() a successful call to run_program() will transfer
	 * control to the new program and never return - if we get
	 * here then run_program() has failed in some way.  It may
	 * just be that we don't have read perms on the binary (common
	 * in OpenServer natives) and just try to exec it.
	 */
	execve(*argv, argv, envp);

	fatal(errno, "program load failed");

	return 1;
}
