/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<sys/mman.h>
#include	"lxrun.h"
#include	"linux/types.h"
#include	"linux/mman.h"

/*
 * mmap() related system calls
 */

/*
 *	mmap
 *	munmap
 *	mprotect
 *	msync		(NOT IMPLEMENTED)
 *	mlock		(NOT IMPLEMENTED)
 *	munlock		(NOT IMPLEMENTED)
 *	mlockall	(NOT IMPLEMENTED)
 *	munlockall	(NOT IMPLEMENTED)
 */

void *
lx_mmap(unsigned long *av)
{
	void	*addr;
	size_t	len;
	int	lx_prot;
	int	lx_flags;
	int	fd;
	off_t	off;
	int	prot	= 0;
	int	flags	= 0;
	void	*r;

	if (! verify_read(av, 6 * sizeof(av[0])))
		return (void *)-1;

	addr	= (void *)av[0];
	len	= (size_t)av[1];
	lx_prot	= (int)av[2];
	lx_flags= (int)av[3];
	fd	= (int)av[4];
	off	= (off_t)av[5];

	if (lx_prot & LX_PROT_READ)
		prot |= PROT_READ;
	if (lx_prot & LX_PROT_WRITE)
		prot |= PROT_WRITE;
	if (lx_prot & LX_PROT_EXEC)
		prot |= PROT_EXEC;

	if (lx_flags & LX_MAP_SHARED)
		flags |= MAP_SHARED;
	if (lx_flags & LX_MAP_PRIVATE)
		flags |= MAP_PRIVATE;
	if (lx_flags & LX_MAP_FIXED)
		flags |= MAP_FIXED;
	
	if (lx_flags & LX_MAP_ANONYMOUS)
		fd = open("/dev/zero", O_RDWR);

	r = mmap(addr, len, prot, flags, fd, off);

	if (lx_flags & LX_MAP_ANONYMOUS)
		close(fd);

	return r;
}	

int
lx_munmap(void *addr, size_t len)
{
	return munmap(addr, len);
}	

int
lx_mprotect(void *addr, size_t len, int lx_prot)
{
	int	prot	= 0;

	if (lx_prot & LX_PROT_READ)
		prot |= PROT_READ;
	if (lx_prot & LX_PROT_WRITE)
		prot |= PROT_WRITE;
	if (lx_prot & LX_PROT_EXEC)
		prot |= PROT_EXEC;

#ifdef OSR5
        /* This is Bela's hack to get around a bug in mprotect(). */
        /* Contributed by S.G. Aug 26 1997 */

        if(mprotect(addr, len, (prot==PROT_NONE)
                                        ? PROT_READ : PROT_NONE) < 0 &&
                                        errno != ENOMEM)
                return -1;

#endif

	return mprotect(addr, len, prot);
}

int lx_msync()		{ errno = ENOSYS; return -1; }	
int lx_mlock()		{ errno = ENOSYS; return -1; }	
int lx_munlock()	{ errno = ENOSYS; return -1; }	
int lx_mlockall()	{ errno = ENOSYS; return -1; }	
int lx_munlockall()	{ errno = ENOSYS; return -1; }	

