/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<string.h>
#include	<sys/msg.h>
#include	"lxrun.h"
#include	"linux/msg.h"

static int
copy_in_msqid_ds(struct lx_msqid_ds *from, struct msqid_ds *to)
{
	if (! from)
	{
		errno = EFAULT;
		return -1;
	}

	memset(to, 0, sizeof(*to));

	to->msg_perm.key	= from->msg_perm.key;
	to->msg_perm.uid	= from->msg_perm.uid;
	to->msg_perm.gid	= from->msg_perm.gid;
	to->msg_perm.cuid	= from->msg_perm.cuid;
	to->msg_perm.cgid	= from->msg_perm.cgid;
	to->msg_perm.mode	= from->msg_perm.mode;
	to->msg_perm.seq	= from->msg_perm.seq;

	to->msg_stime		= from->msg_stime;
	to->msg_rtime		= from->msg_rtime;
	to->msg_ctime		= from->msg_ctime;
	to->msg_cbytes		= from->msg_cbytes;
	to->msg_qnum		= from->msg_qnum;
	to->msg_qbytes		= from->msg_qbytes;
	to->msg_lspid		= from->msg_lspid;
	to->msg_lrpid		= from->msg_lrpid;

	return 0;
}


static int
copy_out_msqid_ds(struct msqid_ds *from, struct lx_msqid_ds *to)
{
	if (! to)
	{
		errno = EFAULT;
		return -1;
	}

	memset(to, 0, sizeof(*to));

	to->msg_perm.key	= from->msg_perm.key;
	to->msg_perm.uid	= from->msg_perm.uid;
	to->msg_perm.gid	= from->msg_perm.gid;
	to->msg_perm.cuid	= from->msg_perm.cuid;
	to->msg_perm.cgid	= from->msg_perm.cgid;
	to->msg_perm.mode	= from->msg_perm.mode;
	to->msg_perm.seq	= from->msg_perm.seq;

	to->msg_stime		= from->msg_stime;
	to->msg_rtime		= from->msg_rtime;
	to->msg_ctime		= from->msg_ctime;
	to->msg_cbytes		= from->msg_cbytes;
	to->msg_qnum		= from->msg_qnum;
	to->msg_qbytes		= from->msg_qbytes;
	to->msg_lspid		= from->msg_lspid;
	to->msg_lrpid		= from->msg_lrpid;

	return 0;
}

int
lx_msgget(key_t key, int msgflg)
{
	return(msgget(key, msgflg));
}

int
lx_msgsnd(int msqid, struct lx_msgbuf *msgp, int msgsz, int msgflg)
{
	return(msgsnd(msqid, msgp, msgsz, msgflg));
}

int
lx_msgrcv(int msqid, struct lx_msgbuf *msgp, int msgsz, long msgtyp, int msgflg)
{
	return(msgrcv(msqid, msgp, msgsz, msgtyp, msgflg));
}

int
lx_msgctl(int msqid, int cmd, struct lx_msqid_ds *buf)
{
	register int err;
	struct msqid_ds sco_buf;

	switch(cmd) {

	case LX_IPC_RMID:
		return(msgctl(msqid, cmd, (struct msqid_ds *)0));

	case LX_IPC_SET:
		copy_in_msqid_ds(buf, &sco_buf);
		return(msgctl(msqid, cmd, &sco_buf));

	case LX_IPC_STAT:
		if ((err = msgctl(msqid, cmd, &sco_buf) == 0))
			copy_out_msqid_ds(&sco_buf, buf);
		return(err);

	default:
		errno = EINVAL;
		return(-1);
	}
}

