/*
 * Copyright (C) 1997 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<string.h>
#include	"lxrun.h"

extern char	*linux_root;

/*
 * safe getstring() for fetching pathnames from the
 * program being run under emulation - checks that
 * each page is readable before copying
 * returns total number of bytes copied (including the
 * null terminator) or -1 in the case of a fault
 */

int
getstring(const void *src, void *dst, int len)
{
	register const char *s = (const char *)src;
	register       char *d = (char *)dst;

	while (len > 0)
	{
		register int n;

		if (getbyte(s) < 0)	/* check for faults */
			return -1;

		n    = PAGE_SIZE - ((unsigned long)src & (PAGE_SIZE-1));
		len -= n;

		while (--n >= 0)
			if (! (*d++ = *s++))
				return d - (char *)dst;
	}
	
	return d - (char *)dst;
}

/*
 * map Linux path names into system path names
 * (mainly for the benefit of the dynamic linker
 * when attempting to load shared libraries)
 */
int
lx_pathname(const char *lx_path, char *os_path, int len)
{
	int	n = getstring(lx_path, os_path, len);
	int	r;

	if (n < 0) {
		errno = EFAULT;
		return -1;
	}

	if (n == len && os_path[n-1] != '\0') {
		errno = ENAMETOOLONG;
		return -1;
	}

	/*
	 * simple hack for now - just trap accesses to /lib and /usr/lib
	 */

	if (linux_root && (strncmp(os_path, "/lib/", 5) == 0 ||
		strncmp(os_path, "/usr/lib/", 9) == 0 ||
		strncmp(os_path, "/usr/X11R6/lib/", 15) == 0) ) {

		r = strlen(linux_root);
		if (r + n > len) {
			errno = ENAMETOOLONG;
			return -1;
		}
		memmove(os_path+r, os_path, n);
		memcpy(os_path, linux_root, r);

		return n + r;
	}

	return n;
}
