/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<fcntl.h>
#include	<limits.h>
#include	<stdlib.h>
#include	<string.h>
#include	<unistd.h>
#include	<sys/mman.h>
#include	<sys/time.h>
#include	<sys/resource.h>
#include	"lxrun.h"
#include	"linux/types.h"
#include	"linux/resource.h"


/*
 * process control system calls
 */

/*
 *	execve
 *	ptrace		(NOT IMPLEMENTED)
 *	brk
 *	setrlimit
 *	getrlimit
 *	getrusage	(NOT IMPLEMENTED)
 *	getgroups
 *	setgroups
 *	getpriority	(NOT IMPLEMENTED)
 *	setpriority	(NOT IMPLEMENTED)
 *	uselib
 *	personality
 *      sched_setparam          (NOT IMPLEMENTED)
 *      sched_getparam          (NOT IMPLEMENTED)
 *      sched_setscheduler      (NOT IMPLEMENTED)
 *      sched_getscheduler      (NOT IMPLEMENTED)
 *      sched_yield             (NOT IMPLEMENTED)
 *      sched_get_priority_max  (NOT IMPLEMENTED)
 *      sched_get_priority_min  (NOT IMPLEMENTED)
 *      sched_rr_get_interval   (NOT IMPLEMENTED)
 */

int
lx_execve(char *prog, char **argv, char **envp)
{
	if (is_linux_exec(prog))
	{
		int	ac	= 0;
		char	**av;
		extern char	emulator[];

                /*
                 * count args, including terminating NULL
                 */
		while (argv[ac++])
			continue;

                /*
                 * allocate new argv and copy old args
                 */
		av	= alloca((ac+1) * sizeof(char *));
		memcpy(&av[1], &argv[0], ac * sizeof(char *));
		av[1]   = prog;
		av[0]   = emulator;
		prog    = emulator;
		argv	= av;
	}

	return execve(prog, argv, envp);
}


int
lx_ptrace()		
{
	errno = ENOSYS;
	return -1;
}	

void *
lx_brk(caddr_t brk)
{
	unsigned long	new_brk;
	unsigned long	old_brk;
	int		z;
	extern caddr_t	minimum_brk;
	extern caddr_t	current_brk;

	if (brk < minimum_brk)
		return current_brk;

	new_brk = PAGE_ALIGN(brk);
	old_brk	= PAGE_ALIGN(current_brk);

	if (new_brk == old_brk)
		return current_brk = brk;

	if (new_brk < old_brk)
	{
		munmap((caddr_t)new_brk, old_brk - new_brk);
		return current_brk = brk;
	}

	if ((z = open("/dev/zero", O_RDWR)) < 0)
	{
		fatal(errno, "can't open /dev/zero");
		return current_brk;
	}


	if (mmap((caddr_t)old_brk, new_brk - old_brk,
		PROT_READ|PROT_WRITE|PROT_EXEC,
		MAP_FIXED|MAP_PRIVATE, z, 0) == (caddr_t)old_brk)
	{
		current_brk = brk;
	}


	close(z);

	return current_brk;
}


int lx_setrlimit(int resource, const struct rlimit *rlp)	/* S.G. */
{
int rsrc;

switch(resource)
	{
	/* Cases that map directly from Linux to SCO */
	case LX_RLIMIT_CPU:	rsrc = RLIMIT_CPU; break;
	case LX_RLIMIT_FSIZE:	rsrc = RLIMIT_FSIZE; break;
	case LX_RLIMIT_DATA:	rsrc = RLIMIT_DATA; break;
	case LX_RLIMIT_STACK:	rsrc = RLIMIT_STACK; break;
	case LX_RLIMIT_CORE:	rsrc = RLIMIT_CORE; break;
	case LX_RLIMIT_NOFILE:	rsrc = RLIMIT_NOFILE; break;

	/* Cases that don't.  :-(  */
	case LX_RLIMIT_RSS:
	case LX_RLIMIT_NPROC:
	default:
		errno = ENOSYS;
		return -1;
	}

return setrlimit(rsrc, rlp);
}


int lx_getrlimit(int resource, struct rlimit *rlp)	/* S.G. */
{
int rsrc;

switch(resource)
	{
	/* Cases that map directly from Linux to SCO */
	case LX_RLIMIT_CPU:	rsrc = RLIMIT_CPU; break;
	case LX_RLIMIT_FSIZE:	rsrc = RLIMIT_FSIZE; break;
	case LX_RLIMIT_DATA:	rsrc = RLIMIT_DATA; break;
	case LX_RLIMIT_STACK:	rsrc = RLIMIT_STACK; break;
	case LX_RLIMIT_CORE:	rsrc = RLIMIT_CORE; break;
	case LX_RLIMIT_NOFILE:	rsrc = RLIMIT_NOFILE; break;

	/* Cases that don't.  :-(  */
	case LX_RLIMIT_RSS:
	case LX_RLIMIT_NPROC:
	default:
		errno = ENOSYS;
		return -1;
	}

return getrlimit(rsrc, rlp);
}


int lx_getrusage()	{ errno = ENOSYS; return -1; }	


int
lx_getgroups(int gidsetsize, lx_gid_t *lx_grouplist)
{
	gid_t	grouplist[NGROUPS_MAX];
	int	r;

	if (gidsetsize < 0 || gidsetsize > NGROUPS_MAX)
	{
		errno = EINVAL;
		return -1;
	}


	r = getgroups(gidsetsize, grouplist);

	if (r > 0 && gidsetsize > 0)
	{
		int	i;

		if (! verify_write(lx_grouplist, r * sizeof(lx_grouplist[0])))
			return -1;

		for (i = 0; i < r; i++)
			lx_grouplist[i] = (lx_gid_t) grouplist[i];
	}

	return r;
}	


int
lx_setgroups(int gidsetsize, lx_gid_t *lx_grouplist)
{
	gid_t	grouplist[NGROUPS_MAX];
	int	i;

	if (gidsetsize < 0 || gidsetsize > NGROUPS_MAX)
	{
		errno = EINVAL;
		return -1;
	}

	if (! verify_read(lx_grouplist, gidsetsize * sizeof(lx_grouplist[0])))
		return -1;

	for (i = 0; i < gidsetsize; i++)
		grouplist[i] = (gid_t) lx_grouplist[i];

	return setgroups(gidsetsize, grouplist);
}


int
lx_getpriority()
{
	errno = ENOSYS;
	return -1;
}	



int
lx_setpriority()
{
	errno = ENOSYS;
	return -1;
}	


int
lx_uselib(char *libname)
{
	char path[PATH_MAX];

	if (lx_pathname(libname, path, PATH_MAX) < 0)
		return -1;

	if (load_library(path) != (void *) -1)
		return 0;

	return -1;
}	


int
lx_personality(unsigned long personality)
{
	if (personality == 0xffffffff || personality == 0)
		return 0;
		
	errno = EINVAL;
	return -1;
}

int lx_sched_setparam()         { errno = ENOSYS; return -1; }
int lx_sched_getparam()         { errno = ENOSYS; return -1; }
int lx_sched_setscheduler()     { errno = ENOSYS; return -1; }
int lx_sched_getscheduler()     { errno = ENOSYS; return -1; }
int lx_sched_yield()            { errno = ENOSYS; return -1; }
int lx_sched_get_priority_max() { errno = ENOSYS; return -1; }
int lx_sched_get_priority_min() { errno = ENOSYS; return -1; }
int lx_sched_rr_get_interval()  { errno = ENOSYS; return -1; }

