/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/shm.h>
#include	"lxrun.h"
#include	"linux/shm.h"


static int
copy_in_shmid_ds(struct lx_shmid_ds *from, struct shmid_ds *to)
{
	if (! from || !verify_read(from, sizeof(*from)))
		return -1;

	memset(to, 0, sizeof(*to));

	to->shm_perm.key	= from->shm_perm.key;
	to->shm_perm.uid	= from->shm_perm.uid;
	to->shm_perm.gid	= from->shm_perm.gid;
	to->shm_perm.cuid	= from->shm_perm.cuid;
	to->shm_perm.cgid	= from->shm_perm.cgid;
	to->shm_perm.mode	= from->shm_perm.mode;
	to->shm_perm.seq	= from->shm_perm.seq;

	to->shm_segsz		= from->shm_segsz;
	to->shm_atime		= from->shm_atime;
	to->shm_dtime		= from->shm_dtime;
	to->shm_ctime		= from->shm_ctime;
	to->shm_cpid		= from->shm_cpid;
	to->shm_lpid		= from->shm_lpid;
	to->shm_nattch		= from->shm_nattch;

	return 0;
}


static int
copy_out_shmid_ds(struct shmid_ds *from, struct lx_shmid_ds *to)
{
	if (! to || !verify_write(to, sizeof(*to)))
		return -1;

	memset(to, 0, sizeof(*to));

	to->shm_perm.key	= from->shm_perm.key;
	to->shm_perm.uid	= from->shm_perm.uid;
	to->shm_perm.gid	= from->shm_perm.gid;
	to->shm_perm.cuid	= from->shm_perm.cuid;
	to->shm_perm.cgid	= from->shm_perm.cgid;
	to->shm_perm.mode	= from->shm_perm.mode;
	to->shm_perm.seq	= from->shm_perm.seq;

	to->shm_segsz		= from->shm_segsz;
	to->shm_atime		= from->shm_atime;
	to->shm_dtime		= from->shm_dtime;
	to->shm_ctime		= from->shm_ctime;
	to->shm_cpid		= from->shm_cpid;
	to->shm_lpid		= from->shm_lpid;
	to->shm_nattch		= from->shm_nattch;

	return 0;
}


int
lx_shmctl(int shmid, int cmd, struct lx_shmid_ds *buf)
{
	int		r	= -1;
	struct shmid_ds	shm_data;

	switch (cmd)
	{
		case LX_IPC_SET:
			if ((r = copy_in_shmid_ds(buf, &shm_data)) == 0)
				r = shmctl(shmid, IPC_SET, &shm_data);
			break;

		case LX_IPC_STAT:
			if ((r = shmctl(shmid, IPC_STAT, &shm_data)) == 0)
				r = copy_out_shmid_ds(&shm_data, buf);
			break;

		case LX_IPC_RMID:
			r = shmctl(shmid, IPC_RMID, NULL);
			break;

		case LX_SHM_LOCK:
			r = shmctl(shmid, SHM_LOCK, NULL);
			break;

		case LX_SHM_UNLOCK:
			r = shmctl(shmid, SHM_UNLOCK, NULL);
			break;

		/*
		 * these are non-standard functions used by
		 * the Linux ipcs(1) program
		 */
		case LX_IPC_INFO:	/* NOT IMPLEMENTED */
		case LX_SHM_INFO:	/* NOT IMPLEMENTED */
		case LX_SHM_STAT:	/* NOT IMPLEMENTED */
			errno = EINVAL;
			break;

		default:
			errno = EINVAL;
			break;
	}

	return r;
}

int
lx_shmget(key_t key, int size, int lx_flag)
{
	int	flag	= lx_flag & 0777;

	if (lx_flag & LX_IPC_CREAT)
		flag	|= IPC_CREAT;
	if (lx_flag & LX_IPC_EXCL)
		flag	|= IPC_EXCL;

	return shmget(key, size, flag);
}

int
lx_shmat(int shmid, char *shmaddr, int lx_shmflg, void **result)
{
	int	shmflg;

	if (lx_shmflg & LX_SHM_RDONLY)
		shmflg	|= SHM_RDONLY;
	if (lx_shmflg & LX_SHM_RND)
		shmflg	|= SHM_RND;
	if (lx_shmflg & LX_SHM_REMAP)
	{
		errno = EINVAL;
		return -1;
	}

	*result = shmat(shmid, shmaddr, shmflg);

	return errno ? -1 : 0;
}

int
lx_shmdt(char *shmaddr)
{
	return shmdt(shmaddr);
}
