/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<sys/time.h>
#include	<sys/socket.h>
#include	"lxrun.h"
#include	"linux/socket.h"

/*
 * OpenServer 5.0 socket function prototypes use int instead of size_t
 */
#if defined(OSR5)
#define	SIZE_T	int
#else
#define	SIZE_T	size_t
#endif


/*
 * sockets
 */
int
lx_old_select(unsigned long *av)
{
	if (! verify_read(av, 5 * sizeof(av[0])))
		return -1;

	return select((int)av[0], (fd_set *)av[1], (fd_set *)av[2],
		(fd_set *)av[3], (struct timeval *)av[4]);
}

int
lx_select(int n, fd_set *rfd, fd_set *wfd, fd_set *xfd, struct timeval *t)
{
	return select(n, rfd, wfd, xfd, t);
}

int
lx_syslog()
{
	errno = ENOSYS;
	return -1;
}

int
lx_socket(int lx_domain, int lx_type, int lx_protocol)
{
	int	domain, type, protocol;

	switch (lx_domain)
	{
		case LX_AF_UNSPEC:	domain = AF_UNSPEC;	break;
		case LX_AF_UNIX:	domain = AF_UNIX;	break;
		case LX_AF_INET:	domain = AF_INET;	break;
		case LX_AF_AX25:
		case LX_AF_IPX:
		default:		errno  = EINVAL;	return -1;
	}

	switch (lx_type)
	{
		case LX_SOCK_STREAM:	type = SOCK_STREAM;	break;
		case LX_SOCK_DGRAM:	type = SOCK_DGRAM;	break;
		case LX_SOCK_RAW:	type = SOCK_RAW;	break;
		case LX_SOCK_RDM:	type = SOCK_RDM;	break;
		case LX_SOCK_SEQPACKET:	type = SOCK_SEQPACKET;	break;
		default:		errno  = EINVAL;	return -1;
	}

	switch (lx_protocol)
	{
		case LX_PF_UNSPEC:	protocol = PF_UNSPEC;	break;
		case LX_PF_UNIX:	protocol = PF_UNIX;	break;
		case LX_PF_INET:	protocol = PF_INET;	break;
		case LX_PF_AX25:
		case LX_PF_IPX:
		default:		protocol = lx_protocol;	break;
	}

	return socket(domain, type, protocol);
}

int
lx_setsockopt(int s, int level, int optname, void *optval, SIZE_T optlen)
{
	/* S.G. { */

	int sco_opt;

	if(optname == LX_SO_DEBUG)		sco_opt= SO_DEBUG;
	else if(optname == LX_SO_REUSEADDR)	sco_opt= SO_REUSEADDR;
	else if(optname == LX_SO_DONTROUTE)	sco_opt= SO_DONTROUTE;
	else if(optname == LX_SO_BROADCAST)	sco_opt= SO_BROADCAST;
	else if(optname == LX_SO_KEEPALIVE)	sco_opt= SO_KEEPALIVE;
	else if(optname == LX_SO_OOBINLINE)	sco_opt= SO_OOBINLINE;
	else if(optname == LX_SO_LINGER)	sco_opt= SO_LINGER;
	else if(optname == LX_SO_SNDBUF)	sco_opt= SO_SNDBUF;
	else if(optname == LX_SO_RCVBUF)	sco_opt= SO_RCVBUF;
	else					sco_opt= -1;

	if(sco_opt == -1 || level != LX_SOL_SOCKET)	/* not implemented */
		{
		errno = EINVAL;
		return -1;
		}

	return setsockopt(s, SOL_SOCKET, sco_opt, optval, optlen);

	/* } S.G. */
}


int
lx_getsockopt(int s, int level, int optname, void *optval, SIZE_T *optlen)
{
	errno = EINVAL;
	return -1;
}

int
lx_socketcall(int call, unsigned long *av)
{
	switch (call) 
	{
		case LX_SYS_SOCKET:
			return lx_socket((int)av[0], (int)av[1], (int)av[2]);

		case LX_SYS_BIND:
			return bind((int)av[0], (struct sockaddr *)av[1],
				(int)av[2]);

		case LX_SYS_CONNECT:
			return connect((int)av[0], (struct sockaddr *)av[1],
				(SIZE_T)av[2]);

		case LX_SYS_LISTEN:
			return listen((int)av[0], (int)av[1]);

		case LX_SYS_ACCEPT:
			return accept((int)av[0],(struct sockaddr *)av[1],
				(SIZE_T *)av[2]);

		case LX_SYS_GETSOCKNAME:
			return getsockname((int)av[0], (struct sockaddr *)av[1],
				(SIZE_T *)av[2]);

		case LX_SYS_GETPEERNAME:
			return getpeername((int)av[0], (struct sockaddr *)av[1],
				(SIZE_T *)av[2]);

		case LX_SYS_SOCKETPAIR:
			return socketpair((int)av[0], (int)av[1], (int)av[2],
				(int *)av[3]);

		case LX_SYS_SEND:
			return send((int)av[0], (void *)av[1], (SIZE_T)av[2],
				(int)av[3]);

		case LX_SYS_SENDTO:
			return sendto((int)av[0], (void *)av[1], (SIZE_T)av[2],
				(int)av[3], (struct sockaddr *)av[4],
				(SIZE_T)av[5]);

		case LX_SYS_RECV:
			return recv((int)av[0], (void *)av[1], (SIZE_T)av[2],
				(int)av[3]);

		case LX_SYS_RECVFROM:
			return recvfrom((int)av[0], (void *)av[1],
				(SIZE_T)av[2], (int)av[3],
				(struct sockaddr *)av[4], (SIZE_T *)av[5]);

		case LX_SYS_SHUTDOWN:
			return shutdown((int)av[0], (int)av[1]);

		case LX_SYS_SETSOCKOPT:
			return lx_setsockopt((int)av[0], (int)av[1], (int)av[2],
				(void *)av[3], (SIZE_T)av[4]);

		case LX_SYS_GETSOCKOPT:
			return lx_getsockopt((int)av[0], (int)av[1], (int)av[2],
				(void *)av[3], (SIZE_T *)av[4]);
	}

	errno = EINVAL;
	return -1;
}
