/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<sys/stat.h>
#include	"lxrun.h"
#include	"linux/stat.h"

/*
 * stat(), fstat(), lstat()
 *
 * only thing done here is to copy out the appropriate structure layout
 *
 * values in the st_mode field should be identical
 * values in the st_dev  field will need to be fixed in dev_t is not 16 bits
 *
 * only potential problem is if a program calls stat() with a file
 * descriptor returned from socket() and expects to see a file type
 * of S_IFSOCK (0140000) - ignore this for now ...
 */

static int
copy_out_old_stat(struct stat *from, struct lx_old_stat *to)
{
	if (! verify_write(to, sizeof(*to)))
		return -1;

	to->st_dev	= from->st_dev;
	to->st_ino	= from->st_ino;
	to->st_mode	= from->st_mode;
	to->st_nlink	= from->st_nlink;
	to->st_uid	= from->st_uid;
	to->st_gid	= from->st_gid;
	to->st_rdev	= from->st_rdev;
	to->st_size	= from->st_size;
	to->st__atime	= from->st_atime;
	to->st__mtime	= from->st_mtime;
	to->st__ctime	= from->st_ctime;

	return 0;
}

static int
copy_out_new_stat(struct stat *from, struct lx_new_stat *to)
{
	if (! verify_write(to, sizeof(*to)))
		return -1;

	to->st_dev	= from->st_dev;
	to->st_ino	= from->st_ino;
	to->st_mode	= from->st_mode;
	to->st_nlink	= from->st_nlink;
	to->st_uid	= from->st_uid;
	to->st_gid	= from->st_gid;
	to->st_rdev	= from->st_rdev;
	to->st_size	= from->st_size;
	to->st_blksize	= from->st_blksize;
	to->st_blocks	= from->st_blocks;
	to->st__atime	= from->st_atime;
	to->st__mtime	= from->st_mtime;
	to->st__ctime	= from->st_ctime;

	return 0;
}

int
lx_oldstat(char *lx_path, struct lx_old_stat *buf)
{
	char		os_path[PATH_MAX];
	int		r;
	struct stat	s;

	if (lx_pathname(lx_path, os_path, PATH_MAX) < 0)
		return -1;

	if ((r = stat(os_path, &s)) == 0)
		r = copy_out_old_stat(&s, buf);

	return r;
}	

int
lx_oldfstat(int fd, struct lx_old_stat *buf)
{
	int		r;
	struct stat	s;

	if ((r = fstat(fd, &s)) == 0)
		r = copy_out_old_stat(&s, buf);

	return r;
}	

int
lx_oldlstat(char *lx_path, struct lx_old_stat *buf)
{
	char		os_path[PATH_MAX];
	int		r;
	struct stat	s;

	if (lx_pathname(lx_path, os_path, PATH_MAX) < 0)
		return -1;

	if ((r = lstat(os_path, &s)) == 0)
		r = copy_out_old_stat(&s, buf);

	return r;
}	

int
lx_stat(char *lx_path, struct lx_new_stat *buf)
{
	char		os_path[PATH_MAX];
	int		r;
	struct stat	s;

	if (lx_pathname(lx_path, os_path, PATH_MAX) < 0)
		return -1;

	if ((r = stat(os_path, &s)) == 0)
		r = copy_out_new_stat(&s, buf);

	return r;
}	

int
lx_fstat(int fd, struct lx_new_stat *buf)
{
	int		r;
	struct stat	s;

	if ((r = fstat(fd, &s)) == 0)
		r = copy_out_new_stat(&s, buf);

	return r;
}	

int
lx_lstat(char *lx_path, struct lx_new_stat *buf)
{
	char		os_path[PATH_MAX];
	int		r;
	struct stat	s;

	if (lx_pathname(lx_path, os_path, PATH_MAX) < 0)
		return -1;

	if ((r = lstat(os_path, &s)) == 0)
		r = copy_out_new_stat(&s, buf);

	return r;
}	
