/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<assert.h>
#include	<errno.h>
#include	<setjmp.h>
#include	<signal.h>
#include	<stdlib.h>
#include	<sys/siginfo.h>
#include	<sys/regset.h>
#include	<sys/reg.h>
#include	<ucontext.h>
#include	"lxrun.h"
#include	"sysent.h"

typedef union
{
	void	*p;
	int	i;
} reg_t;

#define	ARG0	regs[EBX].i
#define	ARG1	regs[ECX].i
#define	ARG2	regs[EDX].i
#define	ARG3	regs[ESI].i
#define	ARG4	regs[EDI].i


extern void deliver_signal(int sig, siginfo_t *info, void *context);

/*
 * the following four declarations are actually all labels
 * in the text segment but they are declared here as extern
 * arrays of unsigned char to emphasise the fact that they are
 * not callable functions
 */
extern uchar_t getbyte_can_fault_here[];
extern uchar_t putbyte_can_fault_here[];
extern uchar_t getbyte_fault[];
extern uchar_t putbyte_fault[];

void
syscall_handler(
	int		sig,
	siginfo_t	*info,
	void		*context
	)
{
	reg_t		*regs;
	uchar_t		*eip;

	assert(sig == SIGSEGV && context != NULL);

#if defined(OSR5)
	regs	= (reg_t *)((ucontext_t *)context)->uc_mcontext.regs;
#else
	regs	= (reg_t *)((ucontext_t *)context)->uc_mcontext.gregs;
#endif
	eip	= regs[EIP].p;

	if (eip == getbyte_can_fault_here)
	{
		regs[EIP].p = getbyte_fault;
	}
	else if (eip == putbyte_can_fault_here)
	{
		regs[EIP].p = putbyte_fault;
	}
	else if (getbyte(eip) == 0xcd && getbyte(eip+1) == 0x80)
	{
		int	n;
		int	rval = -1;

		regs[EIP].i	+= 2;
		n		= regs[EAX].i;	/* system call #	*/
		errno		= 0;

		if (n < 0 || n >= nsysent)
			errno = ENOSYS;
		else
			rval = sysent[n].syscall(ARG0, ARG1, ARG2, ARG3, ARG4);

		regs[EAX].i = (rval == -1) ? - lx_errno(errno) : rval;

#if defined(TRACE)
		trace_syscall(n, &regs[0].i, rval, errno);
#endif
	}
	else
	{
#if defined(TRACE)
		unsigned long	*sp	= regs[UESP].p;

		tprintf("caught signal 11 at EIP = %#X\n", eip);
		tprintf("ESP = %08x, EBP = %08x, ESI = %08x, EDI = %08x\n",
			regs[UESP], regs[EBP], regs[ESI], regs[EDI]);
		tprintf("EAX = %08x, EBX = %08x, ECX = %08x, EDX = %08x\n",
			regs[EAX], regs[EBX], regs[ECX], regs[EDX]);

		tprintf("%08x %08x %08x %08x\n",sp[-4], sp[-3], sp[-2], sp[-1]);

	
		trace_stack(regs[EBP].p);
		exit(1);
#endif
		deliver_signal(sig, info, context);
	}
}
