/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#if defined(TRACE)

#include	<errno.h>
#include	<stdarg.h>
#include	<stdio.h>
#include	<string.h>
#include	<unistd.h>
#include	<sys/time.h>
#include	<sys/reg.h>
#include	"lxrun.h"
#include	"sysent.h"
#include	"linux/ipc.h"
#include	"linux/socket.h"
#include	"trace.h"

#define	ARG0	regs[EBX]
#define	ARG1	regs[ECX]
#define	ARG2	regs[EDX]
#define	ARG3	regs[ESI]
#define	ARG4	regs[EDI]

#define	SYS(scall, rtype, args)	{ #scall, 0, rtype, { args } },

struct trace_info syscall_info[] =
{
#include "syscalls.h"
};

struct trace_info socket_info[] =
{
	SYS(socketcall,		RET(int), ARGS1(int))
	SYS(socket,		RET(int), ARGS3(int, int, int))
	SYS(bind,		RET(int), ARGS3(int, ptr, int))
	SYS(connect,		RET(int), ARGS3(int, ptr, int))
	SYS(listen,		RET(int), ARGS2(int, int))
	SYS(accept,		RET(int), ARGS3(int, ptr, ptr))
	SYS(getsockname,	RET(int), ARGS3(int, ptr, ptr))
	SYS(getpeername,	RET(int), ARGS3(int, ptr, ptr))
	SYS(socketpair,		RET(int), ARGS4(int, int, int, ptr))
	SYS(send,		RET(int), ARGS4(int, ptr, int, int))
	SYS(sendto,		RET(int), ARGS6(int, ptr, int, int, ptr, int))
	SYS(recv,		RET(int), ARGS4(int, ptr, int, int))
	SYS(recvfrom,		RET(int), ARGS6(int, ptr, int, int, ptr, ptr))
	SYS(shutdown,		RET(int), ARGS2(int, int))
	SYS(setsockopt,		RET(int), ARGS5(int, int, int, ptr, int))
	SYS(getsockopt,		RET(int), ARGS5(int, int, int, ptr, ptr))
};

struct trace_info ipc_info[] =
{
	SYS(ipc,		RET(int), ARGS1(int))
	SYS(semop,		RET(int), ARGS3(int, ptr, int))
	SYS(semget,		RET(int), ARGS3(int, int, ptr))
	SYS(semctl,		RET(int), ARGS4(int, int, hex, ptr))
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(msgsnd,		RET(int), ARGS4(int, ptr, int, ptr))
	SYS(msgrcv,		RET(int), ARGS5(int, ptr, int, ptr, ptr))
	SYS(msgget,		RET(int), ARGS2(int, int))
	SYS(msgctl,		RET(int), ARGS3(int, int, ptr))
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(shmat,		RET(int), ARGS4(int, ptr, int, hex))
	SYS(shmdt,		RET(int), ARGS1(ptr))
	SYS(shmget,		RET(int), ARGS3(int, int, hex))
	SYS(shmctl,		RET(int), ARGS3(int, int, hex))
};

int	trace_flag = 1;
FILE	*trace_fp;
int	pid = -1;

int
trace_init(int argc, char *argv[])
{
    return 1;
}

/*
 * tprintf(format, ...)
 */
void
tprintf(
	char	*fmt,		/* message format string		*/
	...			/* optional arguments			*/
	)
{
	va_list	args;

	if (trace_fp == NULL)
	{
		char trace_name[PATH_MAX];
		sprintf(trace_name, "/tmp/lxrun.%d", getpid());
		trace_fp = fopen(trace_name, "w");
		setbuf(trace_fp, NULL);
	}
	va_start(args, fmt);
	(void) vfprintf(trace_fp, fmt, args);
	va_end(args);
}

void
trace_syscall(int n, int *regs, int rval, int err)
{
	struct	sysent	*s;
	int	args[8];
	char	buf[256];
	char	*p;

	args[0]	= ARG0;
	args[1]	= ARG1;
	args[2]	= ARG2;
	args[3]	= ARG3;
	args[4]	= ARG4;
	args[5]	= 0;
	args[6]	= 0;
	args[7]	= 0;

	if (pid == -1)
		pid = getpid();

	if (n == 2)	/* fork() */
	{
		int	newpid = getpid();
#if 0
		if (newpid != pid)
			trace_flag = 0;
#endif
		pid = newpid;
	}

	if (! trace_flag)
		return;
	
	if (n >= 0 && n < nsysent)
	{
		int	i;
		struct trace_info	*t;
		t = &syscall_info[n];
		sprintf(buf, "%s(", t->name);
		p = &buf[strlen(buf)];

		for (i = 0; i < 6 && t->argtypes[i] != 0;)
		{
			if (i > 0)
				strcat(buf, ", ");
			i += PRINT_ARG(t, buf, args, i);

		}
		strcat(buf, ")");
		if (rval == -1) {
			strcat(buf, " ");
			strcat(buf, strerror(err));
		}
		else {
			strcat(buf, " returned ");
			PRINT_RET(t, buf, rval);
		}
		tprintf("%s\n", buf);

		if (t->argtypes[0] == a_path)
		{
			char	os_path[PATH_MAX];

    			lx_pathname((char *)args[0], os_path, PATH_MAX);

			if (strcmp((char *)args[0], os_path) != 0)
			{
				tprintf("*** path mapped to '%.60s'\n", os_path);
			}
		}

	}
	else
	{
		tprintf(buf, "syscall(%d)\n", n);
	}

}

void
trace_signal(int sig, void *info, void *context)
{
	tprintf("pid %d: signal(%d, %#X, %#X)\n", pid, sig, info,
		context);
}

struct stackframe
{
    struct stackframe	*fp;		/* next stack frame	*/
    unsigned char	*ret;		/* return address	*/
    unsigned		arg[1];		/* function arguments	*/
};


static void
prname(unsigned long addr)
{
    if (addr == 0)
	tprintf("????????");
    else
	tprintf("%8x", addr);
}

/*
 * prframe()	- try to print out the contents of a stackframe
 *
 *		   understands 4 types of call:
 *			   e8 xx xx xx xx	call	xx xx xx xx
 *			ff 15 xx xx xx xx	call	ds:[xx xx xx xx]
 *			         ff 55 xx	call	ss:[ebp + xx]
 *			ff 95 xx xx xx xx	call	ss:[ebp + xx xx xx xx]
 */
static void
prframe(struct stackframe *fp)
{
    unsigned char	*r;
    unsigned long	f;
    int			ac;
    int			i;

    r = fp->ret;

    if (r[-5] == 0xe8)
	f = (unsigned long)r + *(long *)(r - 4);
    else if (r[-6] == 0xff && r[-5] == 0x15)
	f = **(unsigned long **)(r - 4);
    else if (r[-6] == 0xff && r[-5] == 0x95)
	f = *(unsigned long *)((unsigned char *)fp->fp + *(long *)(r - 4));
    else if (r[-3] == 0xff && r[-2] == 0x55)
	f = *(unsigned long *)((unsigned char *)fp->fp + *(char *)(r - 1));
    else
	f = 0;
    
    prname(f);		/* print function name	*/
    tprintf("(");

    if (r[0] == 0x83 && r[1] == 0xc4)		/* add esp, imm8	*/
	ac = r[2] / 4;
    else if (r[0] == 0x81 && r[1] == 0xc4)	/* add esp, imm32	*/
	ac = *(unsigned long *)&r[2] / 4;
    else if (r[0] == 0x59)			/* pop ecx		*/
	for (ac = 0; r[ac] == 0x59; ac++)
	    ;
    else
	ac = 0;

    for (i = 0; i < ac; i++)
	tprintf("%x%s", fp->arg[i], (i == ac - 1) ? "" : ", ");

    tprintf(")\t called from ");
    prname((unsigned long)r);
    tprintf("\n");

}


void
trace_stack(void *frame)
{
    struct stackframe	*fp	= frame;

    for (fp = fp->fp; fp->fp != NULL; fp = fp->fp)
	prframe(fp);

    tprintf("\n");
}
#endif
